# 
# 	Sebastien.Letort@toulouse.inra.fr
#	Created: April 05, 2007
#	Last Updated: April 16, 2007
#

package Colors::Shader;

=pod

=head1 NAME

Colors::Shader - a class to generate lists of shaded colors

=head1 SYNOPSIS

	$o_blue = New Colors::Color(__blue => 255);
	$o_shader = New Colors::Shader($o_blue)
	@a_shaded_blue = $o_shader->FromWhiteToBlack();

=head1 DESCRIPTION

	this class allows to create list of shaded colors.

=head1 FILES

	Color.pm => this class is composed with Color

=head1 AUTHOR

Letort Sebastien : Sebastien.Letort@toulouse.inra.fr


=cut


use warnings;
use strict;

use General qw(:DEFAULT &ShowDebug);
use LipmError::ParameterException;

#uniquement pour tester les nombres et le type
use Scalar::Util;
use List::Util qw[min max];

# pour gerer les couleurs facilement
use Colors::Color;
# 0 <= color < 256
use constant MIN	=>	Colors::Color::MIN;
use constant MAX	=>	Colors::Color::MAX;



BEGIN
{
    our $VERSION = do {my @r = (q$Rev$ =~ /\d+/g); $r[0]};
}

=head2 function New

	Title      :	New
	Usage      :	$o_color = New Colors::Shader($o_color[, $step])
	Prerequiste:	none
	Function   :	constructor of the object
	Returns    :	an object
	Args       :	$o_color, a valid color object
                 	$step, optional, an integer,
                 	the difference between 2 successives shades, default = 1
	Globals    :	none

=cut

sub _Init
 {
	my $self = shift;
	my ($o_color, $step) = @_;

# 	$$self{__err} = &FALSE;

	# test des parametres
	if ( !Scalar::Util::blessed($o_color)
			or ( !$o_color->isa("Colors::Color") ) )
	{
		throw LipmError::ParameterException($o_color, 'type',
								-should_be => "a color object",
								-text => "your first argument is not a color object.\n");
	}
	else
	{
		$$self{__o_color} = $o_color;

		if ( DefinedNumber($step) )
		{	$$self{__step} = $step;	}
		else
		{	$$self{__step} = 1;	}
	}

	return;
}

=head2 Procedure SetStep

	Title      :	SetStep
	Usage      :	$o_color->SetStep($step)
	Prerequiste:	none
	Procedure   :	setter to the step attribute
	Returns    :	nothing
	Args       :	$step,	a integer, the step between 2 shades for the color
	Globals    :	none

=cut

sub SetStep
{
	my $self = shift;
	my ($step) = @_;

	if (Scalar::Util::looks_like_number($step))
	{	$$self{__step} = $step;	}
	else
	{	print STDERR "SetStep : $step is not a number\n";	}

	return;
}

=head2 Function Lighten

	Title      :	Lighten
	Usage      :	@a_shades = $o_color->Lighten([$o_color_max])
	Prerequiste:	none
	Function   :	construct a list of shade from color to lighter color
                	without going past the max color. Default is white.
                	Values for RGB are increased by step, but never equals RGB max.
	Returns    :	an array of colors
	Args       :	optionnal $o_color_max,	RGB max that will not be equal (default -MAX, MAX, MAX)
	Globals    :	none

=cut

sub Lighten
{
	my $self = shift;
	my ($o_stop) = @_;

# 	my @a_colors = ();
	my (@a_colors, $r_max, $g_max, $b_max);

	if ( Scalar::Util::blessed($o_stop)
			and ( $o_stop->isa("Colors::Color") ) )
	{	($r_max, $g_max, $b_max) = $o_stop->GetRGB();	}
# 	if( !defined($o_stop) or !$o_stop->isa(&Colors::Color) )
	else
	{	($r_max, $g_max, $b_max) = (&MAX, &MAX, &MAX);	}

	my ($red, $green, $blue) = $$self{__o_color}->GetRGB();

# 	# ajustment of max values
	$r_max -= $$self{__step};
	$g_max -= $$self{__step};
	$b_max -= $$self{__step};

	# boolean that indicate if we can continue
	my $t_red = my $t_green = my $t_blue = &TRUE;

	#calul des nuances
# 	while( ($red < $r_max) or ($green < $g_max) or ($blue < $b_max) )
	while( $t_red or $t_green or $t_blue )
	{
		push( @a_colors, New Colors::Color(__red=>$red, __green=>$green, __blue=>$blue) );
		if($red < $r_max)
		{	$red += $$self{__step};	}
		else
		{	$t_red = &FALSE;	}

		if($green < $g_max)
		{	$green += $$self{__step};	}
		else
		{	$t_green = &FALSE;	}

		if($blue < $b_max)
		{	$blue += $$self{__step};	}
		else
		{	$t_blue = &FALSE;	}
	}

	return @a_colors;
}

=head2 Function Darken

	Title      :	Darken
	Usage      :	@a_shades = $o_color->Darken([$o_color_min])
	Prerequiste:	none
	Function   :	construct a list of shade from color to darker color
                	without going past the min color. Default is black.
                	Values for RGB are decreased by step, but never go past RGB min.
	Returns    :	an array of colors
	Args       :	optionnal $o_color_min,	RGB min that will go past (default -MIN, MIN, MIN)
	Globals    :	none

=cut

sub Darken
{
	my $self = shift;
	my ($o_stop) = @_;

	my @a_colors = ();
	my ($r_min, $g_min, $b_min);

	if( !defined($o_stop) or !$o_stop->isa(&Colors::Color) )
	{	($r_min, $g_min, $b_min) = (&MIN, &MIN, &MIN);	}
	else
	{	($r_min, $g_min, $b_min) = $o_stop->GetRGB();	}

	my ($red, $green, $blue) = $$self{__o_color}->GetRGB();

	# ajustment of min values
	$r_min += $$self{__step};
	$g_min += $$self{__step};
	$b_min += $$self{__step};

	# boolean that indicate if we can continue
	my $t_red = my $t_green = my $t_blue = &TRUE;

	#calcul des nuances
# 	while( ($red >= $r_min) or ($green >= $g_min) or ($blue >= $b_min) )
	while( $t_red or $t_green or $t_blue )
	{
		push( @a_colors, New Colors::Color(__red=>$red, __green=>$green, __blue=>$blue) );

		if($red >= $r_min)
		{	$red -= $$self{__step};	}
		else
		{	$t_red = &FALSE;	}

		if($green >= $g_min)
		{	$green -= $$self{__step};	}
		else
		{	$t_green = &FALSE;	}

		if($blue >= $b_min)
		{	$blue -= $$self{__step};	}
		else
		{	$t_blue = &FALSE;	}
	}

	return @a_colors;
}

=head2 Function FromLightToDark

	Title      :	FromLightToDark
	Usage      :	@a_shades = $o_color->FromLightToDark($nb_shades)
	Prerequiste:	none
	Function   :	construct a list of shades for the current color from white to black
	Returns    :	an array with at least $nb_shades colors
	Args       :	$nb_shades,	the minimum number of shades in the list
	Globals    :	none

=cut

sub FromLightToDark
{
	my $self = shift;
	my ($nb_shades) = @_;
	my @a_colors = ();

	# store the previous step
	my $tmp = $$self{__step};

	if( DefinedNumber($nb_shades) )
	{
		$nb_shades >>= 1;
		#autre solution pour avoir le min sans inclure List::Utils
		#	@colors = sort( $self->{__o_color}->GetRGB() );
		#	$min = $colors[0];
		# sur une liste de 3 nombres �ca passe largement.
		my $num = &MAX - 1 - min($self->{__o_color}->GetRGB());

		$$self{__step} = int($num/$nb_shades);
	}

	# from white to color
	my @a_tmp_list = $self->Lighten();
	shift(@a_tmp_list);	# needed to prevent doublet
	push( @a_colors, reverse(@a_tmp_list) );

	# from color to black
	push ( @a_colors, $self->Darken() );

	# step return to its original value
	$$self{__step} = $tmp;

	return @a_colors;
}

=head2 Function FromDarkToLight

	Title      :	FromDarkToLight
	Usage      :	@a_shades = $o_color->FromDarkToLight()
	Prerequiste:	none
	Function   :	construct a list of shades for the current color from black to white
	Returns    :	an array with colors
	Args       :	none
	Globals    :	none

=cut

sub FromDarkToLight
{
	my $self = shift;
	my ($nb_shades) = @_;
	my @a_colors = ();

	# store the previous step
	my $tmp = $$self{__step};
	if (DefinedNumber($nb_shades) )
	{
		#compute the new step
		$nb_shades >>= 1;
		#autre solution pour avoir le min sans inclure List::Utils
		#	@colors = sort( $self->{__o_color}->GetRGB() );
		#	$min = $colors[0];
		# sur une liste de 3 nombres ca passe largement.
		my $num = &MAX - min($self->{__o_color}->GetRGB());
		$$self{__step} = int($num/$nb_shades);
	}

	# from color to black
	my @a_tmp_list = $self->Darken();
	shift(@a_tmp_list);	# needed to prevent doublet
	push( @a_colors, reverse(@a_tmp_list) );

	# from white to color
	push ( @a_colors, $self->Lighten() );

	# step return to its original value
	$$self{__step} = $tmp;

	return @a_colors;
}


=head2 Function DefinedNumber

	Title      :	DefinedNumber
	Usage      :	$boolean = Colors:DefinedNumber($num)
	Prerequiste:	none
	Function   :	tells if we have a number
	Returns    :	TRUE if $num is defined and if it looks like a number, FALSE otherwise
	Args       :	$num,	the variable to test.
	Globals    :	none

=cut

sub DefinedNumber
{
	my ($num) = @_;

	if ( defined($num) and (Scalar::Util::looks_like_number($num)) )
	{	return &TRUE	}

	return &FALSE;
}

1;
