#
#       Sebastien.Letort@toulouse.inra.fr
#       Created: August 30, 2007
#       Last Updated: August 30, 2007
#       $Id: Enum.pm 473 2009-06-10 15:31:07Z manu $
#
# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

package Enum;

use strict;
use warnings;

=pod

=head1 NAME

	Enum

=head1 SYNOPSIS

	# comme constante !
	my $_o_ALLOW_FIELDS = New Enum( 'un', 'deux', 'trois');
	my $val = 'deux';
	print 'correct'	if( $_o_ALLOW_FIELDS->Includes( $val ) );
	$val = 'quatre';
	print 'inconnu'	if( $_o_ALLOW_FIELDS->Includes( $val ) );

	# data gestion without doublon
	my @a_list = (1..9);
	my $o_enum = New Enum( @a_list );
	$o_enum->AddValues( 3,5,7,11 );
	my @a_no_doublons = $o_enum->ListItems();
	print "@a_no_doublons"; #1 2 3 4 5 6 7 8 9 11

	$o_enum->RemoveValues( 2,4,6,8 ); # list = 1 3 5 7 9 11

=head1 DESCRIPTION

	This is an object way of putting enum data.
	The other way is to use $hash{$val}

=cut


BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 473 $ =~ /\d+/g); $r[0]};
}

use Scalar::Util qw( blessed );

=head2 Function New

 Title        : New
 Usage        : my $o_enum = New Enum( 'un', 'dexu', 'ttt' );
 Prerequisite : none
 Function     : constructor
 Args         : an array of vals, anything
 Globals      : none

=cut

sub New
{
	my $class = shift;
	my $self = {};
	bless( $self, $class );

	foreach my $val (@_)
	{	$self->{ $val } = 1;	}

	return $self;
}

=head2 Function IsOk

 Title        : IsOk
 Usage        : Trust()	if $o_enum->IsOk( $value );
 Prerequisite : none
 Function     : return true if $value is part of the Enum allowed value
 Return       : 1 if $value is part of the Enum values, 0 otherwise
 Args         : $value, anything
 Globals      : none

 Note         : DEPRECATED use Includes

=cut

sub IsOk
{
	my $self = shift;
	my ( $val ) = @_;

	warn( "'IsOk' is deprecated, please change it to 'Includes'\n", caller() );

	return	defined $self->{ $val } ? 1 : 0;
}

=head2 Function Includes

 Title        : Includes
 Usage        : Trust()	if $o_enum->Includes( $value );
 Prerequisite : none
 Function     : return true if $value is part of the Enum allowed value
 Return       : 1 if $value is part of the Enum values, 0 otherwise
 Args         : $value, anything
 Globals      : none

=cut

sub Includes
{
	my $self = shift;
	my ( $val ) = @_;

	return	defined $self->{ $val } ? 1 : 0;
}

=head2 Function ListItems

 Title        : ListItems
 Usage        : my @a_allowed_values = $o_enum->ListItems();
 Prerequisite : none
 Function     : return the values passed to the constructor
 Args         : none
 Return       :	an array of anything
 Globals      : none

=cut

sub ListItems
{
	my $self = shift;

	return	keys %$self;
}


=head2 function AddValues

 Title        : AddValues
 Usage        : $o_enum->AddValues( @a_values );
 Prerequisite : none
 Function     : add new allowed values
 Return       : $self
 Args         : @a_values, a list of values

=cut

sub AddValues
{
	my $self = shift;
	my ( @a_values ) = @_;

	foreach my $value ( @a_values )
	{
		$self->{ $value } = 1;
	}

	return $self;	# devrait être une procedure, chercher si utilise
}


=head2 Procedure RemoveValues

 Title        : RemoveValues
 Usage        : $o_enum->RemoveValues( @a_values );
 Prerequisite : none
 Procedure     : removed allowed values
 Args         : @a_values, a list of values

=cut

sub RemoveValues
{
	my $self = shift;
	my ( @a_values ) = @_;

	foreach my $value ( @a_values )
	{
		delete $self->{ $value };
	}

	return;
}


=head2 function

 Title        : Intersec
 Usage        : $o_enum  = $o_xx->Intersec( @a_list );
                $ra_list = Enum::Intersec( $ra_list1, $ra_list2 );
 Prerequisite : none
 Function     : lists items present both in
                   $o_xx and @a_list
				   $ra_list1 and $ra_list2
 Returns      : $o_enum, Enum object or $ra_list in non object mode
 Args         : @a_list, list in object mode
                $ra_list1, $ra_list2, 2 ref list.

=cut

sub Intersec
{
	my $x = shift;
	my $self;
	my @a_list;

	my $flag;

	if( Scalar::Util::blessed( $x ) and $x->isa( 'Enum' ) )
	{
		$self = $x;
		@a_list = @_;
		$flag = 0;
	}
	else
	{
		$self = New Enum( @$x );
		my $ra_ = shift;
		@a_list = @$ra_;
		$flag = 1;
	}

	my @a_both = ();

	foreach my $data ( @a_list )
	{
		push( @a_both, $data )  if( $self->Includes( $data ) );
	}

	return ( 1 == $flag ) ? \@a_both : New Enum( @a_both );
}

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut
1;
