#
#	sebastien.letort@toulouse.fr
#	Created: April 03, 2008
#	$Id: FastaReader.pm 509 2010-01-19 08:06:57Z sletort $
#

# Copyright INRA/CNRS

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr
# Thomas.Faraut@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based interface for analyzing the different levels of genome
# conservations.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

package FastaReader;

=pod

=head1 NAME

 FastaReader - a class to easily read fasta file, especially multi fasta file

=head1  SYNOPSIS

 my $o_fr = New FastaReader( $file );

 # get the 2nd sequence in raw format
 my $ra_headers = $o_fr->GetHeaders();
 my $header     = $$ra_headers[ 1 ];
 my $seq        = $o_fr->GetRawSequence( $header );	# one string, one line

 # your multifasta file contains mamalian sequence
 # species is specified in sequence header with sp=human for example
 # you want to write human fasta sequence into separated files
 my $i = 0;
 my $ra_headers = $o_fr->GetHeaders( 'sp=human' );
 foreach $header ( @$ra_headers )
 {
 	my $fs = new IO::File( "human_$i", 'w' );
 	$seq   = $o_fr->GetFastaSequence( $header );
 	print $fs $seq;
 	++$i;
 }

 # you want all the headers sequences related to glucose, aldose, and enzyme.
 # this information is noted into the header of the sequence.
 @a_headers = $o_fr->GetHeaders( 'glucose|aldose|ase\s' );

=head1 DESCRIPTION

 This class provide easy access to fasta file.
 It is especially designed if you access specific sequence of a multi fasta
 file or want to read those sequences in another order than the reading order.

=head1 NOTE

 GetHeaders method returns sequence in random order.

 Headers do not have "\n" at the end anymore.
 

=head1 BUGS


=cut

use strict;
use warnings;

# use Data::Dumper;
use IO::File;

# use General;
use LipmError::IOException;

use base qw( LipmObject );


=head2 method _Init

 Title        : _Init
 Usage        : my $o_x = New FastaReader( $filename );
 Prerequisite : none
 Fonction     : initialise an instance of FastaReader class.
 Returns      : a valid FastaReader object
 Args         : $filename, filename of a fasta file

=cut

sub _Init
{
	my $self = shift;
	my ( $filename ) = @_;

	throw LipmError( -text => "$filename is a directory\n" ) if( -d $filename );

	my $fh = $self->{ __fh } = new IO::File( $filename, 'r' )
					or throw LipmError::IOException( $filename, 'f' );

	$self->{ __rh_seqs } = {};
	my $rh_seqs = $self->{ __rh_seqs };

	while( my $line = <$fh> )
	{
		next	if( $line !~ /^>/ );
		chomp $line;

		#on a un en tete fasta
		my $header = $line;

		# on passe a la ligne suivante
		my $offset = $fh->getpos();

		# si on a deja une sequence avec le meme nom
		if( defined( $$rh_seqs{ $header } ) )
		{
			my $txt = "you have more than one sequence with the header -$header-\n";
			my $dbg = "As we use a hash to store sequences, we cannot allow two sequences to have the same header. Headers are our keys.\n";
			throw LipmError( -user_msg => $txt, -text => $dbg );
		}

		# tout est ok, on ajoute la sequence
		$$rh_seqs{ $header } = $offset;
	}
	
	$self->{ __ra_headers } = keys %$rh_seqs;

	return;
}


=head2 function GetFastaSequence

 Title        : GetFastaSequence
 Usage        : $seq = $o_xx->GetFastaSequence( $header );
 Prerequisite : none
 Function     : none
 Returns      : $seq, a sequence fasta formatted
 Args         : $header, a header that correspond to only one sequence
 TODO - Throw an exception if $header not defined
=cut

sub GetFastaSequence
{
	my $self = shift;
	my ( $header ) = @_;

	return undef	if( !defined $header );
	my $seq = "";

	#locate the seq in the file
	my $fh = $self->{ __fh };
	my $offset = $self->{ __rh_seqs }{ $header };
# 	$fh->seek( $offset, SEEK_SET );
	$fh->setpos( $offset );
	while( my $line = <$fh> )
	{
		last	if( $line =~ /^>/ );
		$seq .= $line;
	}

	return $seq;
}


=head2 function GetRawSequence

 Title        : GetRawSequence
 Usage        : $seq = $o_xx->GetRawSequence( $header );
 Prerequisite : none
 Function     : read and return the sequence corresponding to the header.
 Returns      : $seq, the sequence, into only one string
 Args         : $header, a header that correspond to only one sequence
 TODO - Throw an exception if $header not defined

=cut

sub GetRawSequence
{
	my $self = shift;
	my ( $header ) = @_;

	return undef	if( !defined $header );
	my $seq = "";

	#locate the seq in the file
	my $fh = $self->{ __fh };
	my $offset = $self->{ __rh_seqs }{ $header };
# 	$fh->seek( $offset, SEEK_SET );	# ne fonctionne pas !
	$fh->setpos( $offset );
	while( my $line = <$fh> )
	{
		last	if( $line =~ /^>/ );
		chomp( $line );
		$seq .= $line;
	}

	return $seq;
}


=head2 function GetHeaders

 Title        : GetHeaders
 Usage        : $ra_headers = $o_xx->GetHeaders( $regexp );
 Prerequisite : none
 Function     : search all headers that match the $regexp.
 Returns      : $ra_headers, a sorted list ref to all headers.
 Args         : $regexp, a regular expression.
                	will be used as ($header =~ /$regexp/);
                if $regexp is undef, all headers are returned.

 Note         : beware if you look for a specific header and if it contains '|'
                be sure to neutralized them with '\' or "\Q$regexp\E"

=cut

sub GetHeaders
{
	my $self = shift;
	my ( $regexp ) = @_;

	my $ra_headers = $self->{ __ra_headers };
	return $ra_headers	if( !defined $regexp );

	# $regexp is defined, search throught the list
	my $ra_returned_headers = [];
	foreach my $header ( @$ra_headers )
	{
		push( @$ra_returned_headers, $header )	if( $header =~ /$regexp/ );
	}

	return $ra_returned_headers;
}

=pod

=head1 COPYRIGHT NOTICE

This software is governed by the CeCILL license - www.cecill.info

=cut

1;
