#!/usr/bin/env perl

package iANT::Feature::Hit;

use strict;
use warnings;
use XML::Twig;
use iANT::Common;
use iANT::feature::hit::hit;
use iANT::Feature::Hit::Property;
use iANT::Feature::Hit::Raw;
use Carp;

=head1 NAME

iANT::Feature::Hit - Feature hit object


=head1 SYNOPSIS

    my %h_feature = (type => 'manual', path => '/path/to/analysis/file');
    my $o_feature = New iANT::Feature(\%h_feature);
    $o_feature->Set('date','14032006');

    my %h_software    = (name => 'blastp', version  => '2.2.6');
    my $o_feature_software = New iANT::Feature::Software(\%h_software);
    $o_feature->SetSoftware($o_feature_software);

    my %h_database    = (name => 'sptr', version  => 'mars2006', comment => "un commentaire");
    my $o_feature_database = New iANT::Feature::Database(\%h_database);
    $o_feature->SetDatabase($o_feature_database);

    my %h_sequence = (id => 'RSc0092', accession => 'RS02279_AA', length => 113, begin => 22, end => 95, strand=> '+' definition => 'proteine transsmembranaire');
    my $o_feature_sequence = New iANT::Feature::Sequence(\%h_sequence);
    $o_feature->SetSequence($o_feature_sequence);

    my %h_hit = (id => 'Q8Y388_RALSO', accession => 'Q8Y388', length => 113, begin => 22, end => 95, definition => 'PROBABLE TRANSMEMBRANE PROTEIN');
    my $o_feature_hit = New iANT::Feature::Hit(\%h_hit);

    my $o_hit_raw = New iANT::Feature::Hit::Raw({content => "> pas de contenu <"});
    $o_feature_hit->SetRaw($o_hit_raw);

    my $o_hit_property = New iANT::Feature::Hit::Property({id => 'score', value => '100'});
    $o_feature_hit->SetProperty($o_hit_property);
    $o_hit_property = New iANT::Feature::Hit::Property({id => 'gap', value => '0'});
    $o_feature_hit->SetProperty($o_hit_property);
    
    my $retrieved_score = $o_feature_hit->GetProperty('score');
    
    $o_feature->SetHit($o_feature_hit);

    $o_feature->Dump();

    print "\n\n\nRaw: ", $o_feature->GetHit->GetRaw->GetContent(),"\n";

    my $o_hit = $o_feature->GetHit();
    print "Hit   ","\n";
    print "      Id:",$o_hit->GetId(),"\n";
    print "      Ac:",$o_hit->Get('accession'),"\n";


=head1 DESCRIPTION

    iANT::Feature::Hit is designed to make it reasonably easy to create, consume or modify a feature hit using the XML syntax:
    
    <feature type="" path="" date="">
        <software name="" version="" comment=""/>
        <database name="" version="" comment=""/>  
        <sequence id="" accession="" length="" definition="" begin="" end=""/ strand="+" frame="1">
        <hit id="" accession="" definition="" length="" begin="" end="" strand="-" frame="1">
                <property id="bit-score" value=""/>
                <property id="length"    value="100"/>
                <property id="mismatch"  value="4"/>
                <raw></raw>
        </hit>
    </feature>


=cut


=head1 PUBLIC METHODS


=pod 


=head3 function New

 Title        : New
 Usage        : New iANT::Feature::Hit;
 Prerequisite : none
 Function     : Create an object iANT::Feature::Hit
 Returns      : An object iANT::Feature::Hit
 Args         : none
 Globals      : none

=cut

BEGIN
{
    our $VERSION = "0.1";

}

sub New
{
    my ($class, $xml_source) = (@_);

    my $self = {
                __id            => "",
                __accession     => "",
                __definition    => "",
                __length        => "",
                __begin         => "",
                __end           => "",
                __strand        => "",
                __frame        => "",
                __o_raw         => "",
                __ra_properties => undef
                };

    if (defined $xml_source)
    {
        $self = &Update($self, $xml_source);
    }
    bless($self, $class);

    return ($self);
}

=pod

=head3 function Update

 Title        : Update
 Usage        : $o_xml_hit->Update($source);
 Prerequisite : none
 Function     : Update attributes of an existing iANT::Feature::Hit object
 Returns      : An object iANT::Feature::Hit
 Args         : $source: xml file | xml string | hashref
 Globals      : none

=cut

sub Update
{
    my ($self, $source) = (@_);
    my $o_xml_hit;

    if (ref($source) =~ /hash/i)
    {
        $self = &__InitializeFromHash($self, $source);
    }
    elsif (($source !~ /\n/) && (-e $source))
    {
        $o_xml_hit = iANT::feature::hit::hit->new(filename => $source);
        $self = &__InitializeFromXml($self, $o_xml_hit);
    }
    else    #raw XML data
    {
        $o_xml_hit = iANT::feature::hit::hit->new(xml => $source);
        $self = &__InitializeFromXml($self, $o_xml_hit);
    }
    return $self;
}

=pod

=head3 function GetXml

 Title        : GetXml
 Usage        : $o_hit->GetXml();
 Prerequisite : none
 Function     : Retrieve a Hit in xml format
 Returns      : An xml string
 Args         : none
 Globals      : none

=cut

sub GetXml
{
    my ($self, $target_element) = @_;
    my $o_xml_hit = iANT::feature::hit::hit->new();

    if (   (!defined $target_element)
        || ($target_element eq 'hit')
        || ($target_element eq '')
        || ($target_element eq 'all'))
    {
        $o_xml_hit->id($self->{__id});
        $o_xml_hit->accession($self->{__accession});
        $o_xml_hit->definition($self->{__definition});
        $o_xml_hit->length($self->{__length});
        $o_xml_hit->begin($self->{__begin});
        $o_xml_hit->end($self->{__end});
        $o_xml_hit->strand($self->{__strand});
        $o_xml_hit->frame($self->{__frame});

    }

    if (   (!defined $target_element)
        || ($target_element eq 'raw')
        || ($target_element eq '')
        || ($target_element eq 'all'))
    {
        if (defined $self->{__o_raw})
        {
            my @a_o_raw;
            my $o_raw     = $self->{__o_raw};
            my $o_xml_raw = $o_raw->GetXml();
            $o_xml_hit->raw($o_xml_raw);
        }
    }

    if (   (!defined $target_element)
        || ($target_element eq 'property')
        || ($target_element eq '')
        || ($target_element eq 'all'))
    {
        if (defined $self->{__ra_properties})
        {
            my @a_o_property;
            my @a_ro_property = @{$self->{__ra_properties}};
            foreach my $o_property (@a_ro_property)
            {
                my $o_xml_property = $o_property->GetXml();
                push(@a_o_property, $o_xml_property);
            }
            $o_xml_hit->property(@a_o_property);
        }
    }
    return $o_xml_hit;
}

=pod

=head3 procedure Dump

 Title        : Dump
 Usage        : $o_definition->Dump();
 Prerequisite : uses iANT::Common::Dump method
 Function     : Dump a Hit in xml format
 Returns      : none
 Args         : $target:-optional; if empty or undef, print on STDOUT
                -if defined, print in the specified file (unix-like mode is mandatory)
 Globals      : none

=cut

sub Dump
{
    my ($self, $target) = (@_);
    &iANT::Common::Dump(@_);
}

=pod
 
=head3 Getters and Setters

=item Get

    Function     : Generic Getter
    Args         : $attribute
    Prerequisite : uses iANT::Common::Get method

=cut

sub Get
{
    my ($self, $attribut) = @_;
    &iANT::Common::Get(@_);
}

=pod

=item Set

    Function    : Generic Setter
    Args        : $attribute
                  $value
    Prerequisite : uses iANT::Common::Set method

=cut

sub Set
{
    my ($self, $attribut, $value) = @_;
    &iANT::Common::Set(@_);
}

=item GetId

    Returns      : Hit id

=cut

sub GetId
{
    my ($self) = (@_);
    $self->Get('id');
}

=item SetId

    Set hit id

=cut

sub SetId
{
    my ($self, $id) = (@_);
    $self->Set('id', $id);
}

=item GetAccession

    Returns      : Hit accession

=cut

sub GetAccession
{
    my ($self) = (@_);
    $self->Get('accession');
}

=item SetAccession

    Set hit accession

=cut

sub SetAccession
{
    my ($self, $accession) = (@_);
    $self->Set('accession', $accession);
}

=item GetDefinition

    Returns      : Hit definition

=cut

sub GetDefinition
{
    my ($self) = (@_);
    $self->Get('definition');
}

=item SetDefinition

    Set hit definition

=cut

sub SetDefinition
{
    my ($self, $definition) = (@_);
    $self->Set('definition', $definition);
}

=item GetLength

    Returns      : Hit length

=cut

sub GetLength
{
    my ($self) = (@_);
    $self->Get('length');
}

=item SetLength

    Set hit length

=cut

sub SetLength
{
    my ($self, $length) = (@_);
    $self->Set('length', $length);
}

=item GetBegin

    Returns      : Hit begin

=cut

sub GetBegin
{
    my ($self) = (@_);
    $self->Get('begin');
}

=item SetBegin

    Set hit begin

=cut

sub SetBegin
{
    my ($self, $begin) = (@_);
    $self->Set('begin', $begin);
}

=item GetEnd

    Returns      : Hit end

=cut

sub GetEnd
{
    my ($self) = (@_);
    $self->Get('end');
}

=item SetEnd

    Set hit end

=cut

sub SetEnd
{
    my ($self, $end) = (@_);
    $self->Set('end', $end);
}

=item GetStrand

    Returns      : Hit strand

=cut

sub GetStrand
{
    my ($self) = (@_);
    $self->Get('strand');
}

=item SetStrand

    Set hit strand

=cut

sub SetStrand
{
    my ($self, $strand) = (@_);
    $self->Set('strand', $strand);
}

=item GetFrame

    Returns      : Hit frame (blast translation frame)

=cut

sub GetFrame
{
    my ($self) = (@_);
    $self->Get('frame');
}

=item SetFrame

    Set hit frame (blast translation frame)

=cut

sub SetFrame
{
    my ($self, $frame) = (@_);
    $self->Set('frame', $frame);
}

=item GetProperties

    Returns      : Hit properties

=cut

sub GetProperties
{
    my ($self) = (@_);
    $self->Get('ra_properties');
}

=item GetProperty

    Returns      : Named Hit property
    Args         : property id

=cut

sub GetProperty
{
    my ($self, $property_id) = (@_);

    my @a_property_values = ();
    if (!(defined $property_id) || ($property_id eq ''))
    {
        confess("GetProperty method needs a property id\n");
    }
    
    foreach my $o_property (@{$self->Get('ra_properties')})
    {
        if ($o_property->Get('id') eq $property_id)
        {
            push (@a_property_values,$o_property->Get('value'));
        }
    }

    my $property_values_number = $#a_property_values + 1;
    if ($property_values_number > 1)
    {
        confess("GetProperty method returns $property_values_number values for $property_id property\n");
    }
    elsif ($property_values_number == 0)
    {
        return '';
    }
    else
    {
        return $a_property_values[0];
    }

}



=item SetProperty

    Set hit property

=cut

sub SetProperty
{
    my ($self, $o_property) = @_;
    my @a_o_properties = ();
    if (defined $self->{__ra_properties} && ($self->{__ra_properties} ne ''))
    {
        @a_o_properties = @{$self->{__ra_properties}};
    }
    push(@a_o_properties, $o_property);
    $self->{__ra_properties} = \@a_o_properties;
    return;
}

=item GetRaw

    Returns      : Hit raw

=cut

sub GetRaw
{
    my ($self) = (@_);
    $self->Get('o_raw');
}

=item SetRaw

    Set hit raw

=cut

sub SetRaw
{
    my ($self, $o_raw) = @_;
    $self->Set('o_raw', $o_raw);
}

=cut



=head1 PRIVATE METHODS

=pod

=head3 function __InitializeFromHash

 Title        : __InitializeFromHash
 Usage        : __InitializeFromHash($o_hit,$rh_attributes);
 Prerequisite : none
 Function     : Fill attributes of an existing hit object from a hash (attr => value, ...)
 Returns      : An object iANT::Feature::Hit
 Args         : $o_hit
                $rh_attributes
 Globals      : none

=cut

sub __InitializeFromHash
{
    my ($self, $rh_attributes) = (@_);
    
    if (!defined $rh_attributes->{o_raw})
    {
        $rh_attributes->{o_raw} = New iANT::Feature::Hit::Raw();
    }

    $self->{__id}         = $rh_attributes->{id}         if (defined $rh_attributes->{id});
    $self->{__accession}  = $rh_attributes->{accession}  if (defined $rh_attributes->{accession});
    $self->{__definition} = $rh_attributes->{definition} if (defined $rh_attributes->{definition});
    $self->{__length}     = $rh_attributes->{length}     if (defined $rh_attributes->{length});
    $self->{__begin}      = $rh_attributes->{begin}      if (defined $rh_attributes->{begin});
    $self->{__end}        = $rh_attributes->{end}        if (defined $rh_attributes->{end});
    $self->{__strand}     = $rh_attributes->{strand}     if (defined $rh_attributes->{strand});
    $self->{__frame}     = $rh_attributes->{frame}       if (defined $rh_attributes->{frame});
    $self->{__o_raw}      = $rh_attributes->{o_raw}      if (defined $rh_attributes->{o_raw});
    $self->{__ra_properties} = $rh_attributes->{ra_properties}
    if ((defined $rh_attributes->{ra_properties}) && (ref($rh_attributes->{ra_properties}) =~ /array/i));
    return $self;
}

=pod

=head3 function __InitializeFromXml

 Title        : __InitializeFromXml
 Usage        : __InitializeFromXml($o_hit,$o_xml_hit);
 Prerequisite : none
 Function     : Fill attributes of an existing hit object from a Class::XML object
 Returns      : An object iANT::Feature::Hit
 Args         : $o_hit
                $o_xml_hit
 Globals      : none

=cut

sub __InitializeFromXml
{
    my ($self, $o_xml_hit) = (@_);
    $self->{__id}         = $o_xml_hit->id         if (defined $o_xml_hit->id);
    $self->{__accession}  = $o_xml_hit->accession  if (defined $o_xml_hit->accession);
    $self->{__definition} = $o_xml_hit->definition if (defined $o_xml_hit->definition);
    $self->{__length}     = $o_xml_hit->length     if (defined $o_xml_hit->length);
    $self->{__begin}      = $o_xml_hit->begin      if (defined $o_xml_hit->begin);
    $self->{__end}        = $o_xml_hit->end        if (defined $o_xml_hit->end);
    $self->{__strand}     = $o_xml_hit->strand     if (defined $o_xml_hit->strand);
    $self->{__frame}     = $o_xml_hit->frame     if (defined $o_xml_hit->frame);

    my $o_hit_raw = New iANT::Feature::Hit::Raw($o_xml_hit->raw);
    $self->{__o_raw} = $o_hit_raw;

    my @a_o_property;
    foreach my $ro_property ($o_xml_hit->property)
    {
        my $o_property     = iANT::feature::hit::property::property->cast($ro_property);
        my $o_hit_property = New iANT::Feature::Hit::Property($o_property);
        push(@a_o_property, $o_hit_property);
    }
    $self->{__ra_properties} = \@a_o_property;
    return $self;

}

=head1 AUTHOR

    Sebastien Carrere (Sebastien.Carrere@toulouse.inra.fr)
    Jerome Gouzy (Jerome.Gouzy@toulouse.inra.fr)

=head1 MANDATORY PERL PACKAGES

    Class::XML
    XML::Twig
    Carp


=head1 SEE ALSO

    iANT::Feature
    iANT::Feature::Software
    iANT::Feature::Database
    iANT::Feature::Sequence
    iANT::Feature::Hit

=cut

1;
