#!/usr/bin/env perl

package iANT::Feature::Sequence;

use strict;
use warnings;
use iANT::Common;
use XML::Twig;
use iANT::feature::sequence::sequence;

=head1 NAME

iANT::Feature::Sequence - Feature sequence object


=head1 SYNOPSIS

    my %h_feature = (type => 'manual', path => '/path/to/analysis/file');
    my $o_feature = New iANT::Feature(\%h_feature);
    $o_feature->Set('date','14032006');

    my %h_software    = (name => 'blastp', version  => '2.2.6');
    my $o_feature_software = New iANT::Feature::Software(\%h_software);
    $o_feature->SetSoftware($o_feature_software);

    my %h_database    = (name => 'sptr', version  => 'mars2006', comment => "un commentaire");
    my $o_feature_database = New iANT::Feature::Database(\%h_database);
    $o_feature->SetDatabase($o_feature_database);

    my %h_sequence = (id => 'RSc0092', accession => 'RS02279_AA', length => 113, begin => 22, end => 95, definition => 'proteine transsmembranaire');
    my $o_feature_sequence = New iANT::Feature::Sequence(\%h_sequence);
    $o_feature->SetSequence($o_feature_sequence);

    my %h_hit = (id => 'Q8Y388_RALSO', accession => 'Q8Y388', length => 113, begin => 22, end => 95, definition => 'PROBABLE TRANSMEMBRANE PROTEIN');
    my $o_feature_hit = New iANT::Feature::Hit(\%h_hit);

    my $o_hit_raw = New iANT::Feature::Hit::Raw({content => "> pas de contenu <"});
    $o_feature_hit->SetRaw($o_hit_raw);

    my $o_hit_property = New iANT::Feature::Hit::Property({id => 'score', value => '100'});
    $o_feature_hit->SetProperty($o_hit_property);
    $o_hit_property = New iANT::Feature::Hit::Property({id => 'gap', value => '0'});
    $o_feature_hit->SetProperty($o_hit_property);

    $o_feature->SetHit($o_feature_hit);

    $o_feature->Dump();

    print "\n\n\nRaw: ", $o_feature->GetHit->GetRaw->GetContent(),"\n";

    my $o_hit = $o_feature->GetHit();
    print "Hit   ","\n";
    print "      Id:",$o_hit->GetId(),"\n";
    print "      Ac:",$o_hit->Get('accession'),"\n";

=head1 DESCRIPTION

    iANT::Feature::Sequence is designed to make it reasonably easy to create, consume or modify a feature sequence using the XML syntax:
    
    <feature type="" path="" date="">
        <software name="" version="" comment=""/>
        <database name="" version="" comment=""/>  
        <sequence id="" accession="" length="" definition="" begin="" end="" strand='+' frame="1"/>
        <hit id="" accession="" definition="" length="" begin="" end="" strand='-' frame="1">
                <property id="bit-score" value=""/>
                <property id="length"    value="100"/>
                <property id="mismatch"  value="4"/>
                <raw></raw>
        </hit>
    </feature>


=cut


=head1 PUBLIC METHODS


=pod 


=head3 function New

 Title        : New
 Usage        : New iANT::Feature::Sequence;
 Prerequisite : none
 Function     : Create an object iANT::Feature::Sequence
 Returns      : An object iANT::Feature::Sequence
 Args         : none
 Globals      : none

=cut

BEGIN
{
    our $VERSION = "0.1";

}

sub New
{
    my ($class, $xml_source) = (@_);

    my $self = {
                __id         => "",
                __accession  => "",
                __definition => "",
                __length     => "",
                __begin      => "",
                __end        => "",
                __strand     => "",
                __frame     => ""
                };

    if (defined $xml_source)
    {
        $self = &Update($self, $xml_source);
    }
    bless($self, $class);

    return ($self);
}

=pod

=head3 function Update

 Title        : Update
 Usage        : $o_xml_sequence->Update($source);
 Prerequisite : none
 Function     : Update attributes of an existing iANT::Feature::Sequence object
 Returns      : An object iANT::Feature::Sequence
 Args         : $source: xml file | xml string | hashref
 Globals      : none

=cut

sub Update
{
    my ($self, $source) = (@_);
    my $o_xml_sequence;

    if (ref($source) =~ /hash/i)
    {
        $self = &__InitializeFromHash($self, $source);
    }
    elsif (($source !~ /\n/) && (-e $source))
    {
        $o_xml_sequence = iANT::feature::sequence::sequence->new(filename => $source);
        $self = &__InitializeFromXml($self, $o_xml_sequence);
    }
    else    #raw XML data
    {
        $o_xml_sequence = iANT::feature::sequence::sequence->new(xml => $source);
        $self = &__InitializeFromXml($self, $o_xml_sequence);
    }
    return $self;
}

=pod

=head3 function GetXml

 Title        : GetXml
 Usage        : $o_sequence->GetXml();
 Prerequisite : none
 Function     : Retrieve a Sequence in xml format
 Returns      : An xml string
 Args         : none
 Globals      : none

=cut

sub GetXml
{
    my ($self, $target_element) = @_;
    my $o_xml_sequence = iANT::feature::sequence::sequence->new();
    if (   (!defined $target_element)
        || ($target_element eq 'sequence')
        || ($target_element eq '')
        || ($target_element eq 'all'))
    {
        $o_xml_sequence->id($self->{__id});
        $o_xml_sequence->accession($self->{__accession});
        $o_xml_sequence->definition($self->{__definition});
        $o_xml_sequence->length($self->{__length});
        $o_xml_sequence->begin($self->{__begin});
        $o_xml_sequence->end($self->{__end});
        $o_xml_sequence->strand($self->{__strand});
        $o_xml_sequence->frame($self->{__frame});

    }

    return $o_xml_sequence;
}

=pod

=head3 procedure Dump

 Title        : Dump
 Usage        : $o_definition->Dump();
 Prerequisite : uses iANT::Common::Dump method
 Function     : Dump a Sequence in xml format
 Returns      : none
 Args         : $target:-optional; if empty or undef, print on STDOUT
                -if defined, print in the specified file (unix-like mode is mandatory)
 Globals      : none

=cut

sub Dump
{
    my ($self, $target) = (@_);
    &iANT::Common::Dump(@_);
}

=pod
 
=head3 Getters and Setters

=item Get

    Function     : Generic Getter
    Args         : $attribute
    Prerequisite : uses iANT::Common::Get method

=cut

sub Get
{
    my ($self, $attribut) = @_;
    &iANT::Common::Get(@_);
}

=pod

=item Set

    Function    : Generic Setter
    Args        : $attribute
                  $value
    Prerequisite : uses iANT::Common::Set method

=cut

sub Set
{
    my ($self, $attribut, $value) = @_;
    &iANT::Common::Set(@_);
}

=item GetId

    Returns      : Sequence id

=cut

sub GetId
{
    my ($self) = (@_);
    $self->Get('id');
}

=item SetId

    Set sequence id

=cut

sub SetId
{
    my ($self, $id) = (@_);
    $self->Set('id', $id);
}

=item GetAccession

    Returns      : Sequence accession

=cut

sub GetAccession
{
    my ($self) = (@_);
    $self->Get('accession');
}

=item SetAccession

    Set sequence accession

=cut

sub SetAccession
{
    my ($self, $accession) = (@_);
    $self->Set('accession', $accession);
}

=item GetDefinition

    Returns      : Sequence definition

=cut

sub GetDefinition
{
    my ($self) = (@_);
    $self->Get('definition');
}

=item SetDefinition

    Set sequence definition

=cut

sub SetDefinition
{
    my ($self, $definition) = (@_);
    $self->Set('definition', $definition);
}

=item GetLength

    Returns      : Sequence length

=cut

sub GetLength
{
    my ($self) = (@_);
    $self->Get('length');
}

=item SetLength

    Set sequence length

=cut

sub SetLength
{
    my ($self, $length) = (@_);
    $self->Set('length', $length);
}

=item GetBegin

    Returns      : Sequence begin

=cut

sub GetBegin
{
    my ($self) = (@_);
    $self->Get('begin');
}

=item SetBegin

    Set sequence begin

=cut

sub SetBegin
{
    my ($self, $begin) = (@_);
    $self->Set('begin', $begin);
}

=item GetEnd

    Returns      : Sequence end

=cut

sub GetEnd
{
    my ($self) = (@_);
    $self->Get('end');
}

=item SetEnd

    Set sequence end

=cut

sub SetEnd
{
    my ($self, $end) = (@_);
    $self->Set('end', $end);
}

=item GetStrand

    Returns      : Sequence strand

=cut

sub GetStrand
{
    my ($self) = (@_);
    $self->Get('strand');
}

=item SetStrand

    Set sequence strand

=cut

sub SetStrand
{
    my ($self, $strand) = (@_);
    $self->Set('strand', $strand);
}

=item GetFrame

    Returns      : Sequence frame (blast translation frame)

=cut

sub GetFrame
{
    my ($self) = (@_);
    $self->Get('frame');
}

=item SetFrame

    Set sequence frame (blast translation frame)

=cut

sub SetFrame
{
    my ($self, $frame) = (@_);
    $self->Set('frame', $frame);
}

=head1 PRIVATE METHODS

=pod

=head3 function __InitializeFromHash

 Title        : __InitializeFromHash
 Usage        : __InitializeFromHash($o_sequence,$rh_attributes);
 Prerequisite : none
 Function     : Fill attributes of an existing sequence object from a hash (attr => value, ...)
 Returns      : An object iANT::Feature::Sequence
 Args         : $o_sequence
                $rh_attributes
 Globals      : none

=cut

sub __InitializeFromHash
{
    my ($self, $rh_attributes) = (@_);
    $self->{__id}         = $rh_attributes->{id}         if (defined $rh_attributes->{id});
    $self->{__accession}  = $rh_attributes->{accession}  if (defined $rh_attributes->{accession});
    $self->{__definition} = $rh_attributes->{definition} if (defined $rh_attributes->{definition});
    $self->{__length}     = $rh_attributes->{length}     if (defined $rh_attributes->{length});
    $self->{__begin}      = $rh_attributes->{begin}      if (defined $rh_attributes->{begin});
    $self->{__end}        = $rh_attributes->{end}        if (defined $rh_attributes->{end});
    $self->{__strand}     = $rh_attributes->{strand}     if (defined $rh_attributes->{strand});
    $self->{__frame}      = $rh_attributes->{frame}      if (defined $rh_attributes->{frame});
    return $self;
}

=pod

=head3 function __InitializeFromXml

 Title        : __InitializeFromXml
 Usage        : __InitializeFromXml($o_sequence,$o_xml_sequence);
 Prerequisite : none
 Function     : Fill attributes of an existing sequence object from a Class::XML object
 Returns      : An object iANT::Feature::Sequence
 Args         : $o_sequence
                $o_xml_sequence
 Globals      : none

=cut

sub __InitializeFromXml
{
    my ($self, $o_xml_sequence) = (@_);
    $self->{__id}         = $o_xml_sequence->id         if (defined $o_xml_sequence->id);
    $self->{__accession}  = $o_xml_sequence->accession  if (defined $o_xml_sequence->accession);
    $self->{__definition} = $o_xml_sequence->definition if (defined $o_xml_sequence->definition);
    $self->{__length}     = $o_xml_sequence->length     if (defined $o_xml_sequence->length);
    $self->{__begin}      = $o_xml_sequence->begin      if (defined $o_xml_sequence->begin);
    $self->{__end}        = $o_xml_sequence->end        if (defined $o_xml_sequence->end);
    $self->{__strand}     = $o_xml_sequence->strand     if (defined $o_xml_sequence->strand);
    $self->{__frame}      = $o_xml_sequence->frame      if (defined $o_xml_sequence->frame);

    return $self;
}

=head1 AUTHOR

    Sebastien Carrere (Sebastien.Carrere@toulouse.inra.fr)
    Jerome Gouzy (Jerome.Gouzy@toulouse.inra.fr)

=head1 MANDATORY PERL PACKAGES

    Class::XML
    XML::Twig
    Carp


=head1 SEE ALSO

    iANT::Feature
    iANT::Feature::Software
    iANT::Feature::Database
    iANT::Feature::Sequence
    iANT::Feature::Hit

=cut

1;
