#!/usr/bin/env perl

package iANT::Feature::Software;

use strict;
use warnings;
use XML::Twig;
use iANT::Common;
use iANT::feature::software::software;

=head1 NAME

iANT::Feature::Software - Feature Software object


=head1 SYNOPSIS

    my %h_feature = (type => 'manual', path => '/path/to/analysis/file');
    my $o_feature = New iANT::Feature(\%h_feature);
    $o_feature->Set('date','14032006');

    my %h_software    = (name => 'blastp', version  => '2.2.6');
    my $o_feature_software = New iANT::Feature::Software(\%h_software);
    $o_feature->SetSoftware($o_feature_software);

    my %h_database    = (name => 'sptr', version  => 'mars2006', comment => "un commentaire");
    my $o_feature_database = New iANT::Feature::Database(\%h_database);
    $o_feature->SetDatabase($o_feature_database);

    my %h_sequence = (id => 'RSc0092', accession => 'RS02279_AA', length => 113, begin => 22, end => 95, definition => 'proteine transsmembranaire');
    my $o_feature_sequence = New iANT::Feature::Sequence(\%h_sequence);
    $o_feature->SetSequence($o_feature_sequence);

    my %h_hit = (id => 'Q8Y388_RALSO', accession => 'Q8Y388', length => 113, begin => 22, end => 95, definition => 'PROBABLE TRANSMEMBRANE PROTEIN');
    my $o_feature_hit = New iANT::Feature::Hit(\%h_hit);

    my $o_hit_raw = New iANT::Feature::Hit::Raw({content => "> pas de contenu <"});
    $o_feature_hit->SetRaw($o_hit_raw);

    my $o_hit_property = New iANT::Feature::Hit::Property({id => 'score', value => '100'});
    $o_feature_hit->SetProperty($o_hit_property);
    $o_hit_property = New iANT::Feature::Hit::Property({id => 'gap', value => '0'});
    $o_feature_hit->SetProperty($o_hit_property);

    $o_feature->SetHit($o_feature_hit);

    $o_feature->Dump();

    print "\n\n\nRaw: ", $o_feature->GetHit->GetRaw->GetContent(),"\n";

    my $o_hit = $o_feature->GetHit();
    print "Hit   ","\n";
    print "      Id:",$o_hit->GetId(),"\n";
    print "      Ac:",$o_hit->Get('accession'),"\n";


=head1 DESCRIPTION

    iANT::Feature::Software is designed to make it reasonably easy to create, consume or modify a feature software using the XML syntax:
    
    <feature type="" path="" date="">
        <software name="" version="" comment=""/>
        <database name="" version="" comment=""/>  
        <sequence id="" accession="" length="" definition="" begin="" end=""/>
        <hit id="" accession="" definition="" length="" begin="" end="">
                <property id="bit-score" value=""/>
                <property id="length"    value="100"/>
                <property id="mismatch"  value="4"/>
                <raw></raw>
        </hit>
    </feature>


=cut

=cut


=head1 PUBLIC METHODS


=pod 


=head3 function New

 Title        : New
 Usage        : New iANT::Feature::Software;
 Prerequisite : none
 Function     : Create an object iANT::Feature::Software
 Returns      : An object iANT::Feature::Software
 Args         : none
 Globals      : none

=cut

BEGIN
{
    our $VERSION = "0.1";

}

sub New
{
    my ($class, $xml_source) = (@_);

    my $self = {
                __name    => "",
                __version => "",
                __comment => ""
                };

    if (defined $xml_source)
    {
        $self = &Update($self, $xml_source);
    }
    bless($self, $class);

    return ($self);
}

=pod

=head3 function Update

 Title        : Update
 Usage        : $o_xml_software->Update($source);
 Prerequisite : none
 Function     : Update attributes of an existing iANT::Feature::Software object
 Returns      : An object iANT::Feature::Software
 Args         : $source: xml file | xml string | hashref
 Globals      : none

=cut

sub Update
{
    my ($self, $source) = (@_);
    my $o_xml_software;

    if (ref($source) =~ /hash/i)
    {
        $self = &__InitializeFromHash($self, $source);
    }
    elsif (($source !~ /\n/) && (-e $source))
    {
        $o_xml_software = iANT::feature::software::software->new(filename => $source);
        $self = &__InitializeFromXml($self, $o_xml_software);
    }
    else    #raw XML data
    {
        $o_xml_software = iANT::feature::software::software->new(xml => $source);
        $self = &__InitializeFromXml($self, $o_xml_software);
    }
    return $self;
}

=pod

=head3 function GetXml

 Title        : GetXml
 Usage        : $o_software->GetXml();
 Prerequisite : none
 Function     : Retrieve a Software in xml format
 Returns      : An xml string
 Args         : none
 Globals      : none

=cut

sub GetXml
{
    my ($self, $target_element) = @_;
    my $o_xml_software = iANT::feature::software::software->new();
    if (   (!defined $target_element)
        || ($target_element eq 'software')
        || ($target_element eq '')
        || ($target_element eq 'all'))
    {
        $o_xml_software->name($self->{__name});
        $o_xml_software->version($self->{__version});
        $o_xml_software->comment($self->{__comment});
    }

    return $o_xml_software;
}

=pod

=head3 procedure Dump

 Title        : Dump
 Usage        : $o_definition->Dump();
 Prerequisite : uses iANT::Common::Dump method
 Function     : Dump a Software in xml format
 Returns      : none
 Args         : $target:-optional; if empty or undef, print on STDOUT
                -if defined, print in the specified file (unix-like mode is mandatory)
 Globals      : none

=cut

sub Dump
{
    my ($self, $target) = (@_);
    &iANT::Common::Dump(@_);
}

=pod
 
=head3 Getters and Setters

=item Get

    Function     : Generic Getter
    Args         : $attribute
    Prerequisite : uses iANT::Common::Get method

=cut

sub Get
{
    my ($self, $attribut) = @_;
    &iANT::Common::Get(@_);
}

=pod

=item Set

    Function    : Generic Setter
    Args        : $attribute
                  $value
    Prerequisite : uses iANT::Common::Set method

=cut

sub Set
{
    my ($self, $attribut, $value) = @_;
    &iANT::Common::Set(@_);
}

=item GetName

    Returns      : Software name

=cut

sub GetName
{
    my ($self) = (@_);
    $self->Get('name');
}

=item SetName

    Set Software name

=cut

sub SetName
{
    my ($self, $name) = (@_);
    $self->Set('name', $name);
}

=item GetVersion

    Returns      : Software version

=cut

sub GetVersion
{
    my ($self) = (@_);
    $self->Get('version');
}

=item SetVersion

    Set Software version

=cut

sub SetVersion
{
    my ($self, $version) = (@_);
    $self->Set('version', $version);
}

=item GetComment

    Returns      : Software comment

=cut

sub GetComment
{
    my ($self) = (@_);
    $self->Get('comment');
}

=item SetComment

    Set Software comment

=cut

sub SetComment
{
    my ($self, $comment) = (@_);
    $self->Set('comment', $comment);
}

=cut



=head1 PRIVATE METHODS

=pod

=head3 function __InitializeFromHash

 Title        : __InitializeFromHash
 Usage        : __InitializeFromHash($o_software,$rh_attributes);
 Prerequisite : none
 Function     : Fill attributes of an existing Software object from a hash (attr => value, ...)
 Returns      : An object iANT::Feature::Software
 Args         : $o_software
                $rh_attributes
 Globals      : none

=cut

sub __InitializeFromHash
{
    my ($self, $rh_attributes) = (@_);
    $self->{__name}    = $rh_attributes->{name}    if (defined $rh_attributes->{name});
    $self->{__version} = $rh_attributes->{version} if (defined $rh_attributes->{version});
    $self->{__comment} = $rh_attributes->{comment} if (defined $rh_attributes->{comment});
    return $self;
}

=pod

=head3 function __InitializeFromXml

 Title        : __InitializeFromXml
 Usage        : __InitializeFromXml($o_software,$o_xml_software);
 Prerequisite : none
 Function     : Fill attributes of an existing Software object from a Class::XML object
 Returns      : An object iANT::Feature::Software
 Args         : $o_software
                $o_xml_software
 Globals      : none

=cut

sub __InitializeFromXml
{
    my ($self, $o_xml_software) = (@_);
    $self->{__name}    = $o_xml_software->name    if (defined $o_xml_software->name);
    $self->{__version} = $o_xml_software->version if (defined $o_xml_software->version);
    $self->{__comment} = $o_xml_software->comment if (defined $o_xml_software->comment);
    return $self;
}

=head1 AUTHOR

    Sebastien Carrere (Sebastien.Carrere@toulouse.inra.fr)
    Jerome Gouzy (Jerome.Gouzy@toulouse.inra.fr)

=head1 MANDATORY PERL PACKAGES

    Class::XML
    XML::Twig
    Carp


=head1 SEE ALSO

    iANT::Feature
    iANT::Feature::Software
    iANT::Feature::Database
    iANT::Feature::Sequence
    iANT::Feature::Hit

=cut

1;
