#!/usr/bin/env perl

package iANT::Organism;

use strict;
use warnings;
use Carp;
use XML::Twig;

use iANT::Common;
use iANT::organism::organism;

use iANT::Organism::Species;
use iANT::Organism::Taxonomy;
use iANT::Organism::Organelle;

BEGIN
{
    our $VERSION = "0.1";
}

=head1 NAME

iANT::Organism - iANT/MENS Organism object


=head1 SYNOPSIS





=head1 DESCRIPTION

    iANT::Organism is designed to make it reasonably easy to create, consume or modify iANT or MENS organism following the XML syntax:
    
    <organism>
        <species code="rsoa">Ralstonia solanacearum</species>
        <taxonomy ontology="NCBI" id="taxid">dfsgjklsdflgjsdfklg</taxonomy>
        <organelle/>
    </organism>


=cut

=cut


=head1 PUBLIC METHODS

=pod 

=head3 function New

 Title        : New
 Usage        : New iANT::Organism;
 Prerequisite : none
 Function     : Create an object iANT::Organism
 Returns      : An object iANT::Organism
 Args         : none
 Globals      : none

=cut

sub New
{
    my ($class, $source) = (@_);

    my $self = {
                __o_species   => "",
                __o_taxonomy  => "",
                __o_organelle => ""
                };

    if (defined $source)
    {
        $self = &Update($self, $source);
    }
    else
    {
        $self = &Update($self, {});
    }

    bless($self, $class);

    return ($self);
}

=pod

=head3 function Update

 Title        : Update
 Usage        : $o_organism->Update($source);
 Prerequisite : none
 Function     : Update attributes of an existing Organism object
 Returns      : An object Organism
 Args         : $source: xml file | xml string | hashref
 Globals      : none

=cut

sub Update
{
    my ($self, $source) = (@_);
    my $o_xml_organism;

    if (ref($source) =~ /hash/i)
    {
        $self = &__InitializeFromHash($self, $source);
    }
    elsif (($source !~ /\n/) && (-e $source))
    {
        $o_xml_organism = iANT::organism::organism->new(filename => $source);
        $self = &__InitializeFromXml($self, $o_xml_organism);
    }
    else    #raw XML data
    {
        $o_xml_organism = iANT::organism::organism->new(xml => $source);
        $self = &__InitializeFromXml($self, $o_xml_organism);
    }
    return $self;
}

=pod

=head3 function GetXml

 Title        : GetXml
 Usage        : $o_entry->GetXml($target_element);
 Prerequisite : none
 Function     : Retrieve an Entry (or part of) in xml format
 Returns      : An xml string
 Args         : $target_element: optional; if empty or undef, retrieve all
                xml elements.
 Globals      : none

=cut

sub GetXml
{
    my ($self, $target_element) = @_;
    my $o_xml_organism = iANT::organism::organism->new();
    if (   (!defined $target_element)
        || ($target_element eq 'organism')
        || ($target_element eq '')
        || ($target_element eq 'all'))
    {

    }

    if (   (!defined $target_element)
        || ($target_element eq 'species')
        || ($target_element eq '')
        || ($target_element eq 'all'))
    {
        if (defined $self->{__o_species})
        {
            my $o_species     = $self->{__o_species};
            my $o_xml_species = $o_species->GetXml();
            $o_xml_organism->species($o_xml_species);
        }
    }

    if (   (!defined $target_element)
        || ($target_element eq 'taxonomy')
        || ($target_element eq '')
        || ($target_element eq 'all'))
    {
        if (defined $self->{__o_taxonomy})
        {
            my $o_taxonomy     = $self->{__o_taxonomy};
            my $o_xml_taxonomy = $o_taxonomy->GetXml();
            $o_xml_organism->taxonomy($o_xml_taxonomy);
        }
    }
    if (   (!defined $target_element)
        || ($target_element eq 'organelle')
        || ($target_element eq '')
        || ($target_element eq 'all'))
    {
        if (defined $self->{__o_organelle})
        {
            my $o_organelle     = $self->{__o_organelle};
            my $o_xml_organelle = $o_organelle->GetXml();
            $o_xml_organism->organelle($o_xml_organelle);
        }
    }

    return $o_xml_organism;
}

=pod

=head3 procedure Dump

 Title        : Dump
 Usage        : $o_organism->Dump();
 Prerequisite : uses iANT::Common::Dump method
 Function     : Dump an Organism in xml format
 Returns      : none
 Args         : $target:-optional; if empty or undef, print on STDOUT
                        -if defined, print in the specified file (unix-like mode is mandatory)
 Globals      : none

=cut

sub Dump
{
    my ($self, $target) = (@_);
    &iANT::Common::Dump(@_);
}

=pod
 
=head3 Getters and Setters

=item Get

    Function     : Generic Getter
    Args         : $attribute
    Prerequisite : uses iANT::Common::Get method

=cut

sub Get
{
    my ($self, $attribut) = @_;
    &iANT::Common::Get(@_);
}

=pod

=item Set

    Function    : Generic Setter
    Args        : $attribute
                  $value
    Prerequisite : uses iANT::Common::Set method

=cut

sub Set
{
    my ($self, $attribut, $value) = @_;
    &iANT::Common::Set(@_);

}


=item GetSpecies

    Returns      : Organism::Species object

=cut

sub GetSpecies
{
    my ($self) = (@_);
    $self->Get('o_species');
}

=item SetSpecies

    Set Organism Species object

=cut

sub SetSpecies
{
    my ($self, $o_species) = @_;
    $self->Set('o_species', $o_species);
}

=item GetTaxonomy

    Returns      : Organism::Taxonomy object

=cut

sub GetTaxonomy
{
    my ($self) = (@_);
    $self->Get('o_taxonomy');
}

=item SetTaxonomy

    Set Organism Taxonomy object

=cut

sub SetTaxonomy
{
    my ($self, $o_taxonomy) = @_;
    $self->Set('o_taxonomy', $o_taxonomy);
}

=item GetOrganelle

    Returns      : Organism::Organelle object

=cut

sub GetOrganelle
{
    my ($self) = (@_);
    $self->Get('o_organelle');
}

=item SetOrganelle

    Set Organism Organelle object

=cut

sub SetOrganelle
{
    my ($self, $o_organelle) = @_;
    $self->Set('o_organelle', $o_organelle);
}

=cut



=head1 PRIVATE METHODS

=pod

=head3 function __InitializeFromHash

 Title        : __InitializeFromHash
 Usage        : __InitializeFromHash($o_entry,$rh_attributes);
 Prerequisite : none
 Function     : Fill attributes of an existing Entry object from a hash (attr => value, ...)
 Returns      : An object Entry
 Args         : $o_entry:
                $rh_attributes
 Globals      : none

=cut

sub __InitializeFromHash
{
    my ($self, $rh_attributes) = (@_);
    if (!defined $rh_attributes->{o_species})
    {
        $rh_attributes->{o_species} = New iANT::Organism::Species();
    }
    if (!defined $rh_attributes->{o_taxonomy})
    {
        $rh_attributes->{o_taxonomy} = New iANT::Organism::Taxonomy();
    }
    if (!defined $rh_attributes->{o_organelle})
    {
        $rh_attributes->{o_organelle} = New iANT::Organism::Organelle();
    }

    $self->{__o_species}   = $rh_attributes->{o_species}   if (defined $rh_attributes->{o_species});
    $self->{__o_taxonomy}  = $rh_attributes->{o_taxonomy}  if (defined $rh_attributes->{o_taxonomy});
    $self->{__o_organelle} = $rh_attributes->{o_organelle} if (defined $rh_attributes->{o_organelle});

    return $self;
}

=pod

=head3 function __InitializeFromXml

 Title        : __InitializeFromXml
 Usage        : __InitializeFromXml($o_organism,$o_xml_organism);
 Prerequisite : none
 Function     : Fill attributes of an existing Entry object from a Class::XML object
 Returns      : An object Entry
 Args         : $o_entry
                $o_xml_entry
 Globals      : none

=cut

sub __InitializeFromXml
{
    my ($self, $o_xml_organism) = (@_);

    my $o_organism_species = New iANT::Organism::Species($o_xml_organism->species);
    $self->{__o_species} = $o_organism_species;
    my $o_organism_taxonomy = New iANT::Organism::Taxonomy($o_xml_organism->taxonomy);
    $self->{__o_taxonomy} = $o_organism_taxonomy;
    my $o_organism_organelle = New iANT::Organism::Organelle($o_xml_organism->organelle);
    $self->{__o_organelle} = $o_organism_organelle;

    return $self;
}

=head1 AUTHOR

    Sebastien Carrere (Sebastien.Carrere@toulouse.inra.fr)
    Jerome Gouzy (Jerome.Gouzy@toulouse.inra.fr)

=head1 MANDATORY PERL PACKAGES

    Class::XML
    XML::Twig
    Carp


=head1 SEE ALSO

    iANT::Entry::Definition
    iANT::Entry::Link
    iANT::Entry::Sequence

=cut

1;
