#!/usr/bin/env perl

package iANT::Organism::Species;

use strict;
use warnings;
use XML::Twig;
use iANT::Common;
use iANT::organism::species::species;
use HTML::Entities;

=head1 NAME

iANT::Organism::Species - Organism Species object


=head1 SYNOPSIS



=head1 DESCRIPTION

    iANT::Organism::Species is designed to make it reasonably easy to create, consume or modify iANT or MENS organism species following the XML syntax:
    
    <organism>
        <species code="rsoa">Ralstonia solanacearum</species>
        <taxonomy ontology="NCBI" id="taxid">dfsgjklsdflgjsdfklg</taxonomy>
        <organelle>Plasmid</organelle>
    </organism>


=cut

=cut


=head1 PUBLIC METHODS


=pod 


=head3 function New

 Title        : New
 Usage        : New iANT::Organism::Species;
 Prerequisite : none
 Function     : Create an object iANT::Organism::Species
 Returns      : An object iANT::Organism::Species
 Args         : none
 Globals      : none

=cut

BEGIN
{
    our $VERSION = "0.1";
}

sub New
{
    my ($class, $xml_source) = (@_);

    my $self = {
                __code    => "",
                __strain  => "",
                __content => ""
                };

    if (defined $xml_source)
    {
        $self = &Update($self, $xml_source);
    }
    bless($self, $class);

    return ($self);
}

=pod

=head3 function Update

 Title        : Update
 Usage        : $o_species->Update($source);
 Prerequisite : none
 Function     : Update attributes of an existing iANT::Organism::Species object
 Returns      : An object iANT::Organism::Species
 Args         : $source: xml file | xml string | hashref
 Globals      : none

=cut

sub Update
{
    my ($self, $source) = (@_);
    my $o_xml_species;
    chomp $source;

    if (ref($source) =~ /hash/i)
    {
        $self = &__InitializeFromHash($self, $source);
    }
    elsif (($source !~ /\n/) && (-e $source))
    {
        $o_xml_species = iANT::organism::species::species->new(filename => $source);
        $self = &__InitializeFromXml($self, $o_xml_species);
    }
    else    #raw XML data
    {
        $o_xml_species = iANT::organism::species::species->new(xml => $source);
        $self = &__InitializeFromXml($self, $o_xml_species);
    }
    return $self;
}

=pod

=head3 function GetXml

 Title        : GetXml
 Usage        : $o_species->GetXml();
 Prerequisite : none
 Function     : Retrieve an Species in xml format
 Returns      : An xml string
 Args         : none
 Globals      : none

=cut

sub GetXml
{
    my ($self) = @_;
    my $o_xml_species = iANT::organism::species::species->new();

    $o_xml_species->code($self->{__code});
    $o_xml_species->strain($self->{__strain});

    my $o_twig = XML::Twig->new();
    $o_twig->parse($o_xml_species);
    my $root            = $o_twig->root;
    my $encoded_content = HTML::Entities::encode($self->GetContent());
    $root->set_text($encoded_content);
    $o_xml_species = iANT::organism::species::species->new(xml => $o_twig->sprint());

    return $o_xml_species;
}

=pod

=head3 procedure Dump

 Title        : Dump
 Usage        : $o_definition->Dump();
 Prerequisite : uses iANT::Common::Dump method
 Function     : Dump a an Organelle in xml format
 Returns      : none
 Args         : $target:-optional; if empty or undef, print on STDOUT
                -if defined, print in the specified file (unix-like mode is mandatory)
 Globals      : none

=cut

sub Dump
{
    my ($self, $target) = (@_);
    &iANT::Common::Dump(@_);
}

=pod
 
=head3 Getters and Setters

=item Get

    Function     : Generic Getter
    Args         : $attribute
    Prerequisite : uses iANT::Common::Get method

=cut

sub Get
{
    my ($self, $attribut) = @_;
    &iANT::Common::Get(@_);
}

=pod

=item Set

    Function    : Generic Setter
    Args        : $attribute
                  $value
    Prerequisite : uses iANT::Common::Set method

=cut

sub Set
{
    my ($self, $attribut, $value) = @_;
    &iANT::Common::Set(@_);
}

=item GetCode

    Returns      : Species code

=cut

sub GetCode
{
    my ($self) = @_;
    $self->Get('code');
}

=item SetCode

    Set Species code

=cut

sub SetCode
{
    my ($self, $code) = @_;
    $self->Set('code', $code);
}

=item GetStrain

    Returns      : Species strain

=cut

sub GetStrain
{
    my ($self) = @_;
    $self->Get('strain');
}

=item SetStrain

    Set Species strain

=cut

sub SetStrain
{
    my ($self, $strain) = @_;
    $self->Set('strain', $strain);
}

=item GetContent

    Returns      : Species content

=cut

sub GetContent
{
    my ($self) = @_;
    $self->Get('content');
}

=item SetContent

    Set Species content

=cut

sub SetContent
{
    my ($self, $content) = @_;
    $self->Set('content', $content);
}

=cut



=head1 PRIVATE METHODS

=pod

=head3 function __InitializeFromHash

 Title        : __InitializeFromHash
 Usage        : __InitializeFromHash($o_raw,$rh_attributes);
 Prerequisite : none
 Function     : Fill attributes of an existing species object from a hash (attr => value, ...)
 Returns      : An object iANT::Organism::Species
 Args         : $o_organelle
                $rh_attributes
 Globals      : none

=cut

sub __InitializeFromHash
{
    my ($self, $rh_attributes) = (@_);
    $self->{__code}    = $rh_attributes->{code}    if (defined $rh_attributes->{code});
    $self->{__strain}  = $rh_attributes->{strain}  if (defined $rh_attributes->{strain});
    $self->{__content} = $rh_attributes->{content} if (defined $rh_attributes->{content});
    return $self;
}

=pod

=head3 function __InitializeFromXml

 Title        : __InitializeFromXml
 Usage        : __InitializeFromXml($o_raw,$o_xml_raw);
 Prerequisite : none
 Function     : Fill attributes of an existing species object from a Class::XML object
 Returns      : An object iANT::Organism::Species
 Args         : $o_species
                $o_xml_species
 Globals      : none

=cut

sub __InitializeFromXml
{
    my ($self, $o_xml_species) = (@_);

    $self->{__code}   = $o_xml_species->code   if (defined $o_xml_species->code);
    $self->{__strain} = $o_xml_species->strain if (defined $o_xml_species->strain);

    my $o_twig = XML::Twig->new();
    $o_twig->parse($o_xml_species);
    my $root = $o_twig->root;
    $self->{__content} = HTML::Entities::decode($root->text);
    return $self;
}

=head1 AUTHOR

    Sebastien Carrere (Sebastien.Carrere@toulouse.inra.fr)
    Jerome Gouzy (Jerome.Gouzy@toulouse.inra.fr)

=head1 MANDATORY PERL PACKAGES

    Class::XML
    XML::Twig
    Carp


=head1 SEE ALSO

    iANT::Feature
    iANT::Entry
    

=cut

1;
