#!/usr/bin/env perl

package iANT::Organism::Taxonomy;

use strict;
use warnings;
use XML::Twig;
use iANT::Common;
use iANT::organism::taxonomy::taxonomy;
use HTML::Entities;

=head1 NAME

iANT::Organism::Taxonomy - Organism Taxonomy object


=head1 SYNOPSIS



=head1 DESCRIPTION

    iANT::Organism::Taxonomy is designed to make it reasonably easy to create, consume or modify iANT or MENS organism taxonomy following the XML syntax:
    
    <organism>
        <species code="rsoa">Ralstonia solanacearum</species>
        <taxonomy ontology="NCBI" id="taxid">dfsgjklsdflgjsdfklg</taxonomy>
        <organelle>Plasmid</organelle>
    </organism>


=cut

=cut


=head1 PUBLIC METHODS


=pod 


=head3 function New

 Title        : New
 Usage        : New iANT::Organism::Taxonomy;
 Prerequisite : none
 Function     : Create an object iANT::Organism::Taxonomy
 Returns      : An object iANT::Organism::Taxonomy
 Args         : none
 Globals      : none

=cut

BEGIN
{
    our $VERSION = "0.1";
}

sub New
{
    my ($class, $xml_source) = (@_);

    my $self = {
                __ontology => "",
                __id       => "",
                __content  => ""
                };

    if (defined $xml_source)
    {
        $self = &Update($self, $xml_source);
    }

    bless($self, $class);

    return ($self);
}

=pod

=head3 function Update

 Title        : Update
 Usage        : $o_taxonomy->Update($source);
 Prerequisite : none
 Function     : Update attributes of an existing iANT::Organism::Taxonomy object
 Returns      : An object iANT::Organism::Taxonomy
 Args         : $source: xml file | xml string | hashref
 Globals      : none

=cut

sub Update
{
    my ($self, $source) = (@_);
    my $o_xml_taxonomy;
    chomp $source;

    if (ref($source) =~ /hash/i)
    {
        $self = &__InitializeFromHash($self, $source);
    }
    elsif (($source !~ /\n/) && (-e $source))
    {
        $o_xml_taxonomy = iANT::organism::taxonomy::taxonomy->new(filename => $source);
        $self = &__InitializeFromXml($self, $o_xml_taxonomy);
    }
    else    #raw XML data
    {
        $o_xml_taxonomy = iANT::organism::taxonomy::taxonomy->new(xml => $source);
        $self = &__InitializeFromXml($self, $o_xml_taxonomy);
    }
    return $self;
}

=pod

=head3 function GetXml

 Title        : GetXml
 Usage        : $o_taxonomy->GetXml();
 Prerequisite : none
 Function     : Retrieve an Taxonomy in xml format
 Returns      : An xml string
 Args         : none
 Globals      : none

=cut

sub GetXml
{
    my ($self) = @_;
    my $o_xml_taxonomy = iANT::organism::taxonomy::taxonomy->new();

    $o_xml_taxonomy->ontology($self->{__ontology});
    $o_xml_taxonomy->id($self->{__id});

    my $o_twig = XML::Twig->new();
    $o_twig->parse($o_xml_taxonomy);
    my $root            = $o_twig->root;
    my $encoded_content = HTML::Entities::encode($self->GetContent());
    $root->set_text($encoded_content);
    $o_xml_taxonomy = iANT::organism::taxonomy::taxonomy->new(xml => $o_twig->sprint());

    return $o_xml_taxonomy;
}

=pod

=head3 procedure Dump

 Title        : Dump
 Usage        : $o_definition->Dump();
 Prerequisite : uses iANT::Common::Dump method
 Function     : Dump a an Taxonomy in xml format
 Returns      : none
 Args         : $target:-optional; if empty or undef, print on STDOUT
                -if defined, print in the specified file (unix-like mode is mandatory)
 Globals      : none

=cut

sub Dump
{
    my ($self, $target) = (@_);
    &iANT::Common::Dump(@_);
}

=pod
 
=head3 Getters and Setters

=item Get

    Function     : Generic Getter
    Args         : $attribute
    Prerequisite : uses iANT::Common::Get method

=cut

sub Get
{
    my ($self, $attribut) = @_;
    &iANT::Common::Get(@_);
}

=pod

=item Set

    Function    : Generic Setter
    Args        : $attribute
                  $value
    Prerequisite : uses iANT::Common::Set method

=cut

sub Set
{
    my ($self, $attribut, $value) = @_;
    &iANT::Common::Set(@_);
}

=item GetOntology

    Returns      : Taxonomy ontology

=cut

sub GetOntology
{
    my ($self) = @_;
    $self->Get('ontology');
}

=item SetOntology

    Set Taxonomy ontology

=cut

sub SetCode
{
    my ($self, $ontology) = @_;
    $self->Set('ontology', $ontology);
}

=item GetId

    Returns      : Taxonomy id

=cut

sub GetId
{
    my ($self) = @_;
    $self->Get('id');
}

=item SetId

    Set Taxonomy id

=cut

sub SetId
{
    my ($self, $id) = @_;
    $self->Set('id', $id);
}

=item GetContent

    Returns      : Taxonomy content

=cut

sub GetContent
{
    my ($self) = @_;
    $self->Get('content');
}

=item SetContent

    Set Taxonomy content

=cut

sub SetContent
{
    my ($self, $content) = @_;
    $self->Set('content', $content);
}

=cut



=head1 PRIVATE METHODS

=pod

=head3 function __InitializeFromHash

 Title        : __InitializeFromHash
 Usage        : __InitializeFromHash($o_taxonomy,$rh_attributes);
 Prerequisite : none
 Function     : Fill attributes of an existing taxonomy object from a hash (attr => value, ...)
 Returns      : An object iANT::Organism::Taxonomy
 Args         : $o_taxonomy
                $rh_attributes
 Globals      : none

=cut

sub __InitializeFromHash
{
    my ($self, $rh_attributes) = (@_);
    $self->{__ontology} = $rh_attributes->{ontology} if (defined $rh_attributes->{ontology});
    $self->{__id}       = $rh_attributes->{id}       if (defined $rh_attributes->{id});
    $self->{__content}  = $rh_attributes->{content}  if (defined $rh_attributes->{content});
    return $self;
}

=pod

=head3 function __InitializeFromXml

 Title        : __InitializeFromXml
 Usage        : __InitializeFromXml($o_taxonomy,$o_xml_taxonomy);
 Prerequisite : none
 Function     : Fill attributes of an existing taxonomy object from a Class::XML object
 Returns      : An object iANT::Organism::Taxonomy
 Args         : $o_taxonomy
                $o_xml_taxonomy
 Globals      : none

=cut

sub __InitializeFromXml
{
    my ($self, $o_xml_taxonomy) = (@_);

    $self->{__ontology} = $o_xml_taxonomy->ontology if (defined $o_xml_taxonomy->ontology);
    $self->{__id}       = $o_xml_taxonomy->id       if (defined $o_xml_taxonomy->id);
    my $o_twig = XML::Twig->new();
    $o_twig->parse($o_xml_taxonomy);
    my $root = $o_twig->root;
    $self->{__content} = HTML::Entities::decode($root->text);
    return $self;
}

=head1 AUTHOR

    Sebastien Carrere (Sebastien.Carrere@toulouse.inra.fr)
    Jerome Gouzy (Jerome.Gouzy@toulouse.inra.fr)

=head1 MANDATORY PERL PACKAGES

    Class::XML
    XML::Twig
    Carp


=head1 SEE ALSO

    iANT::Feature
    iANT::Entry
    

=cut

1;
