/*
   $Id: Display.h 649 2011-10-31 13:57:42Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#ifndef __DISPLAY_H__
#define __DISPLAY_H__

#include <stdio.h>
#include <stdlib.h>
#include <list>
#include <stack>

using namespace std;
#include "buffer.h"
#include "ptypes.h"
#include "Parameters.h"
#include "utils.h"
class Hsp;

struct regionFreq
{
	uint32_t 			start;
	uint32_t 			end;
};

/**
 * \brief The class Display is an abstract class, it is the base class of several DisplayXXX classes
 *        - If the constructor parameter "mate" is true, the mate output file is opened
 *        - DisplayTopHeader is called Only once, just after initialization
 *        - DisplayHits is one of the entry points: print a vector pf hsps, grouping them in hits
 *        - DisplayHsp is the second entry point: print a hsp, the print is thus less structured
 *        - _DisplayHitPile is called by DisplayHit to print a HitPile
 *        - _DisplayHeader and _DisplayQueryInfo are called by _DisplayHitPile, before displaying the pile itself
 *        - _DisplaySingleHit is called by _DisplayHitPile
 *        - DisplaySubjectInfo is called by _DisplaySingleHit
 *
*/

class QueryBuffer;
class Hsp;
class HitPile;
class Hit;
class Display {
	public:
	    Display():
		    _query_ptr(NULL),_parameters(Parameters::Instance()){};
		virtual ~Display(){};

		void SetQuery( const QueryBuffer* q ) { _query_ptr = q; };
		virtual void DisplayTopHeader() const {};  // called by Comparison - TODO Arranger ca
	    virtual void DisplayHits(vector<Hsp*>& v,int len=-1) const;
	    virtual void DisplayHsp(const Hsp& h) const = 0;
	    virtual void DisplayQueryInfo(const string& query) const { };
	    virtual void DisplaySubjectInfo(const string& subject,int score) const { };
		virtual void DisplaySuitableFasta(const Buffer* freqBuffer, const stack<regionFreq>& suitableRegions) const {};
			
	protected:
	    virtual void _DisplayHitPile(const HitPile&) const;
	    virtual void _DisplayHeader(const string& query,const string& bank) const { };
	        
	private:
	    void __DisplaySingleHit(const Hit&) const;

	protected:
	    const QueryBuffer*  _query_ptr;
	    const Parameters&   _parameters;
};

class DisplayM0: public Display
{
	public:
	    virtual void DisplayHits(vector<Hsp*>& v,int len=-1) const {};
	    virtual void DisplayHsp(const Hsp& h) const {};
};

class DisplayStd: public Display { 
		class DspFile {
			public:
				DspFile (const Parameters& p, bool mate): f(FOpen((mate?p.GetOutputBwdFilename():p.GetOutputFilename()), "w")) {};
				FILE * Get() const { return f; };
				~DspFile() { FClose(f); };
			private:
				FILE* f;
		};

	public:
		DisplayStd (bool mate=false) : Display (), _dsp_file(_parameters,mate) {};
	
	protected:
		DspFile _dsp_file;
};

		
class DisplayM2: public DisplayStd {
    public:
		DisplayM2(bool mate=false): DisplayStd(mate) {};
	    virtual void DisplayTopHeader() const;
	    virtual void DisplayHsp(const Hsp&) const;
};

class DisplayM3: public DisplayStd {
    public:
		DisplayM3(bool mate=false): DisplayStd(mate) {};
	    virtual void DisplayTopHeader() const;
	    virtual void DisplayHsp(const Hsp&) const;
};

class DisplayM8: public DisplayM3 {
    public:
		DisplayM8(bool mate=false): DisplayM3(mate) {};
	    virtual void DisplayTopHeader() const {};
	    virtual void DisplayHsp(const Hsp&) const;
};

class DisplayM9: public DisplayM8 {
	public:
		DisplayM9(bool mate=false): DisplayM8(mate) {};
	
	protected:
		void _DisplayHeader(const string& query,const string& bank) const;
};

class DisplayM4: public DisplayM2 {
    public:
		DisplayM4(bool mate=false): DisplayM2(mate) {};
		void DisplayHsp(const Hsp&) const;
};

class DisplayBed: public DisplayStd {
	protected:
		virtual void DisplayHsp(const Hsp&) const;
};

class DisplayFasta: public DisplayStd {
	public:
		DisplayFasta(bool mate=false): DisplayStd(mate) {};
		void DisplayHsp(const Hsp&) const;
	    
	protected:
		void _DisplayHitPile(const HitPile&) const;  
};

class DisplayFreq: public DisplayFasta {
	public : 
		void DisplaySuitableFasta(const Buffer* freqBuffer, const stack<regionFreq>& suitableRegions) const;

};

class DisplayFastq: public DisplayStd {
	public:
		DisplayFastq(bool mate=false): DisplayStd(mate) {};
		void DisplayHsp(const Hsp&) const;
	    
	protected:
		void _DisplayHitPile(const HitPile&) const;  
};

#endif
