/*
   $Id: DisplayM1.cpp 649 2011-10-31 13:57:42Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#include <iostream>
using namespace std;

#include "utils.h"
#include "DisplayM1.h"
#include "QueryBuffer.h"

/**
 * \brief AlignDisplayLine is used to format a 3 lines - style hsp representation: Query/Matches/Subject
 *        AddColum is used for adding a column to the line, 3 types may be used (gap for query,subject,or no gap)
 *        When enough coluumns are added, they are printed using __DisplayPrettyAlignmentLine.
 * 
 */
enum ColType {q_gap,no_gap,s_gap};
class AlignDisplayLine {
	public:
		AlignDisplayLine( FILE *f, const Hsp&);
		~AlignDisplayLine() { __DisplayPrettyAlignmentLine(); };
		void SetData(const Hsp&);
		void AddColumn(ColType,char,char);
		
	private:
		mutable uint32_t query_ind, subject_ind;
		char *sincr;
		int8_t strand;
		char alignment[3][DISPLAY_LINE_SIZE];
		FILE* dsp_file;
		void __DisplayPrettyAlignmentLine() const;
		uint32_t __current_column;
		
		// static fileds: they are initialized onyl one time
		static char Incr[];
		static char Decr[];
		static char Rev[];
		static bool __static_initialized;
		static void __InitStaticFields();
};

char AlignDisplayLine::Incr[256];
char AlignDisplayLine::Decr[256];
char AlignDisplayLine::Rev[256];
bool AlignDisplayLine::__static_initialized = false;

/**
 * \brief This function is called by the constructor, it is useful only the first time
 */

void AlignDisplayLine::__InitStaticFields()
{
	if (__static_initialized)
		return;
	
	__static_initialized = true;
	
	for (int i=0;i<256;i++) 
	{
		Incr[i]=1;
		Decr[i]=-1;
		Rev[i] = i;
	}

	Incr['-']=0;

	Decr['-']=0;

	Rev['A']='T';
	Rev['T']='A';
	Rev['C']='G';
	Rev['G']='C';
	Rev['a']='t';
	Rev['t']='a';
	Rev['c']='g';
	Rev['g']='c';
	Rev['N']='N';
}

/**
 * \brief The AlignDisplayLine constructor
 */

AlignDisplayLine::AlignDisplayLine( FILE *f, const Hsp& h):
	query_ind(0),
	subject_ind(0),
	sincr(NULL),
	strand(0),
	dsp_file(f),
	__current_column(0)
{
	__InitStaticFields();
	query_ind = h.GetQueryStart();
	strand    = h.GetStrand();
	if (strand > 0) {
		subject_ind = h.GetSubjectStart();
		sincr       = Incr;
	} else {
		subject_ind = h.GetSubjectEnd();
		sincr       = Decr;
	}

}

/**
 * \brief display 3 lines (Query/matches/Subject)
 */
void AlignDisplayLine::__DisplayPrettyAlignmentLine() const
{
	if ( __current_column == 0 )
		return;
		
	fprintf(dsp_file,"Query:  %-10u   ",query_ind);
	for (uint32_t j=0;j<__current_column;j++) {
		fprintf(dsp_file,"%c",alignment[0][j]);
		query_ind+=Incr[(unsigned short)alignment[0][j]];
	}
	
	fprintf(dsp_file,"   %-10u\n",query_ind-1);
	fprintf(dsp_file,"%21s","");
	for (uint32_t j=0;j<__current_column;j++) {
		fprintf(dsp_file,"%c",alignment[1][j]);
	}
	
	fprintf(dsp_file,"\n");
	fprintf(dsp_file,"Subjt:  %-10u   ",subject_ind);
	for (uint32_t j=0;j<__current_column;j++) {
		fprintf(dsp_file,"%c",alignment[2][j]);
		subject_ind += sincr[(unsigned short)alignment[2][j]];
	}
	
	fprintf(dsp_file,"   %-10u\n",subject_ind - strand);
	fprintf(dsp_file,"\n");
}

/**
 * \brief Add a column. When DISPLAY_LINE_SIZE have been added, a line is printed
 * \param col_type The type of column to add (cf ColType)
 * \param q_base The character to add to the query line (only for s_gap, no_gap)
 * \param s_base The character to add to the subject line (only for q_gap, no_gap)
 */

void AlignDisplayLine::AddColumn(ColType col_type,char q_base,char s_base)
{
	switch (col_type)
	{
		case q_gap:
			alignment[0][__current_column] = '-';
			alignment[1][__current_column] = ' ';
			alignment[2][__current_column] = (strand>0) ? s_base : Rev[(int)s_base];
			break;
		case s_gap:
			alignment[0][__current_column] = q_base;
			alignment[1][__current_column] = ' ';
			alignment[2][__current_column] = '-';
			break;
		case no_gap:
			char s_b = (strand>0) ? s_base : Rev[(int)s_base];
			alignment[0][__current_column] = q_base;
			alignment[1][__current_column] = (toupper(q_base)==toupper(s_b)) ? '|' : ' ';
			alignment[2][__current_column] = s_b;
			break;
	}
	if ( __current_column++ == DISPLAY_LINE_SIZE - 1 )
	{
		__DisplayPrettyAlignmentLine();
		__current_column = 0;
	}
}

#ifdef OLD_STUFF
/**
 * \brief Redefined virtual functions
 */
void DisplayM1::_DisplayHeader(const string& query,const string& bank) const
{
	fprintf( _dsp_file.Get(),"-------------------------------------------\n");
	fprintf( _dsp_file.Get(),"Query %s\n",query.c_str());
	fprintf( _dsp_file.Get(),"Searching bank %s \n",bank.c_str());
	fprintf( _dsp_file.Get(),"-------------------------------------------\n");
}
#endif

void DisplayM1::DisplayQueryInfo(const string& query) const
{
	fprintf(_dsp_file.Get(),"Query=%s\n",query.c_str());
}

void DisplayM1::DisplaySubjectInfo(const string& subject,int score) const
{
	fprintf(_dsp_file.Get(),">%s\t%d\t",subject.c_str(),score);
}

/**
 * \brief display a hsp
 * \param The hsp to display
 */

void DisplayM1::DisplayHsp(const Hsp& hsp) const
{
	if ( _query_ptr == NULL )
		throw ( logic_error ( (string) "ERROR - query unknown !" + ERROR_FILE_LINE(__FILE__,__LINE__)));
		
	if ( __no_bank_mode )
	{
		if ( _query_ptr -> GetSubjectBase() != NULL )
		{
			__bank_seq = _query_ptr -> GetSubjectBase();
		}
		else
		{
			string msg = "ERROR - Subject sequence unknown\n";
			msg       += "You must have the MD auxiliary field specified to convert from SAM/BAM to M1\n";
			msg       += "(you could try the -m samext/bamext switch of glint)";
			throw ( runtime_error ( msg) );
		}
	}

    const char *char_sign=(hsp.GetStrand() >0) ? OUTPUT_PLUS : OUTPUT_MINUS;
	fprintf(_dsp_file.Get(),"Score : %d\tSize : %d\t%s\n%u\t%u\t%u\t%u\n",hsp.GetScore(),hsp.GetSize(),char_sign,hsp.GetQueryStart(),hsp.GetQueryEnd(),hsp.GetSubjectStart(),hsp.GetSubjectEnd());	
	fprintf(_dsp_file.Get(),"Gap : %d  %d \n",hsp.GetQueryGaps().GetSize(),hsp.GetSubjectGaps().GetSize());

	const unsigned char *nQuery = _query_ptr -> GetBase() + hsp.GetBQueryStart();
	const unsigned char *nBase  = (hsp.GetStrand()>0) ? __bank_seq + hsp.GetBSubjectStart() : __bank_seq + hsp.GetBSubjectEnd();
	AlignDisplayLine display_line(_dsp_file.Get(), hsp);

    GapPile_const_iterator qgap = hsp.GetQueryGaps().begin();
    GapPile_const_iterator sgap = hsp.GetSubjectGaps().begin();

	for (uint32_t i=0; i<hsp.GetSize(); i++ ) {
		if (qgap!=hsp.GetQueryGaps().end() && *qgap==i) {
			display_line.AddColumn(q_gap,' ',*nBase);
			nBase+=hsp.GetStrand();
			++qgap;
		} else if (sgap!=hsp.GetSubjectGaps().end() && *sgap==i) {
			display_line.AddColumn(s_gap,*nQuery,' ');
			++sgap;
			nQuery++;
		} else {
			display_line.AddColumn(no_gap,*nQuery,*nBase);
			nBase+=hsp.GetStrand();
			nQuery++;
		}
	}
}
