/*
   $Id: GlintEngine.h 649 2011-10-31 13:57:42Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#ifndef __GLINTENGINE_H__
#define __GLINTENGINE_H__

#include <stdio.h>
#include <stdlib.h>
#include <algorithm>
#include <list>
#include <string.h>
#include <math.h>
#include <limits>
#include <climits>
#include "bitcount.h"
#include "scoring.h"
#include "ptypes.h"
#include "hsp.h"
#include "QueryBuffer.h"
#include "QueryBufferDouble.h"
#include "LowComplexityMasker.h"

#define POW24 0x01000000
#define POW20 0x00100000
#define MIN_DIAG LONG_MIN       /* Attention à la compatibilité 32 bits 46 bits */

#define Strandboo(X) ((X>0) ? 1 : 0)

/**
 * \brief A Match is created when a seed found in the tree matches with the request.
 *        The following is wrapped in the Match:
 *            - The position in the sequence and in the query (spos, qpos)
 *            - The score
 *            - The level of the seed in the tree (c'est hit ?)
 *            - The diagonal of the match: <0, 0, >0
*/

struct match {
    match():diag(0),score(0),hit(0),spos(0),qpos(0){};
	DiagInt diag;
	uint16_t score;
	uint8_t hit;
	uint32_t spos;
	uint32_t qpos;
	friend ostream & operator<<(ostream& os, const match &);
};

/* A very basic ostream operator for match */
inline ostream & operator<<(ostream& os, const match& m)
{
	os<<m.diag<<"\t"<<m.qpos<<"\t"<<m.spos<<"\t"<<m.score<<"\t"<<m.hit;
	return os;
}

class ComparisonBase;
class CompactTree;
class Display;
class AlignEngine;

/** \brief GlintEngineSingleStrand
 *         - This class represents the atomic behavior of the glint process on one strand            
 *  \note WARNING it DOES NOT derive from GlintEngineBase
*/
class GlintEngineSingleStrand
{
	public:
		GlintEngineSingleStrand(int s, const Display& d);
		~GlintEngineSingleStrand() { delete[] p; };
		
		uint32_t Push(uint32_t pos_s, uint32_t q_pos,uint8_t hit, const QueryBuffer&);
		
		void ComputeSimpleMatches(const QueryBuffer& query);
		
		void MatchReset();
		void HspReset(); 
		int GetNbHsp() const  { return __hsp_cnt;};
		int GetStrand() const { return __strand; };

		/* Iterators */
		HspIterator begin();
		HspIterator end();
		
	private:
	
		bool __IsNotEmpty() { return (__top>0); };
		
		void __ScoreSlidingDiagonal(const QueryBuffer& query);
		bool __LocateMatchesChunk(match *m[], int min, int max, int16_t score, const QueryBuffer& query);
		void __QSortSeeds(void);
		
		void DisplaySeeds(void);    //for debugging only
		
		const Parameters& __parameters;
		//const Bank&       __bank;
		
		/* Variables recording information on the previously detected hsp */
		DiagInt LastDiag;   // The last diagonal on which an hsp has been detected
		uint32_t SubjectEnd;   // The subject end on this last diagonal
		uint32_t SubjectStart; // The subject start on this last diagonal

		const int __strand; // The strand of this engine (-1/1)
		
		uint32_t   __top;
		HspPile __HP;
		uint64_t   __hsp_cnt;    // counting the displayed hsps
		
		auto_ptr<AlignEngine> __align_engine_aptr;
		AlignEngine&          __align_engine;
		auto_ptr<AlignEngine> __InitAlignEnginePtr();
		
		const Display& __display;

		/* The score matrix */
		const MatchConstants& __match_scoring;

		/* Counting the bits */
		const BitCount & __bit_count;
	
		match *l[MATCH_PILE_SIZE];
		match *p;	
};

/** \brief GlintEngineBase
 *         - This hierarchy of classes represents the atomic behavior of the glint process   
 *         - NOTE - When the GlintEngine is created from a QueryBuffer*, we DO NOT FREE the queryBuffer at the end:
 *                  the user of the glintengine did create the buffer, it should delete it
 *                  When the GlintEngine is created from a COPY CONSTRUCTOR, then the QueryBuffer is CLONED from the source engine
 *                  Then the engine IS responsible for the QueryBuffer destruction.
 * 					This behavior is controlled by the private variable __query_aptr, different from the protected _query_ptr.
*/

class GlintEngineBase
{
	public:
		GlintEngineBase(QueryBuffer*, Display& d);
		GlintEngineBase(const GlintEngineBase& eng);
		virtual ~GlintEngineBase() {};
		virtual GlintEngineBase* Clone() = 0;
	    
	    virtual void Run();
	    virtual bool ReadNextSeq();
		QueryBuffer* GetQueryPtr() const { return _query_ptr;}; 
	    
	   // virtual void GetMappingInfos() = 0;
	    virtual int GetNbHsp() const = 0; 
		virtual void BuildHsps () = 0 ;			
		virtual void DisplayHsps ();
		virtual void HspReset() = 0; 
		  
	protected:
		Parameters&  _parameters;
		const Mask&  _mask;
		CompactTree& _ctree;
		Display&     _display;
		QueryBuffer* _query_ptr;
		
		uint16_t _nb_besthits;
		uint16_t _nb_subopthits;
		
	protected:	
		int _g_log_n[256];	    
	
	private:		
		void __Run ();	
		virtual void __DisplayAll() = 0;
		void operator=(const GlintEngineBase&) {};
		auto_ptr<QueryBuffer> __query_aptr;	// See the remark above
};

/**
 * \brief the GlintEngine 
 *         - The class represents the atomic behavior of the glint process              
*/
typedef bool(*pt2CmpFunc)(const Hsp*, const Hsp*);

class GlintEngine: public GlintEngineBase
{
	public:
		GlintEngine(QueryBuffer*, Display& d);
		~GlintEngine() {};
		GlintEngine(const GlintEngine& eng);
		virtual GlintEngine* Clone() { return new GlintEngine( *this ); };
						
	    int GetNbHsp() const { return __plus.GetNbHsp() + __minus.GetNbHsp(); };
		void HspReset(){ __plus.HspReset(); __minus.HspReset(); __hspl.clear(); };				
		void BuildHsps ();			
		virtual void DisplayHsps ();
		void RecordHsps();
		vector<Hsp*>& GetHsps() { return __hspl;};
		bool HspEmpty() const { return __hspl.empty(); };
		void SortingHsps(pt2CmpFunc cmp_fn_ptr);
		void GetMappingInfos();
		Hsp* GetBestHsp() { return __hspl[0]; };
		
	protected:
		void __MaskLowComplexity();			
					
		LowComplexityMasker __lowcomplexity_masker;
		
		GlintEngineSingleStrand __plus;
		GlintEngineSingleStrand __minus;
		
	private:
		
		vector<Hsp*> __hspl;
	
		void __ComputeHsp();
		void __SearchingForSeeds();
		void __DisplayAll();	
	    void __Status();
 	    void __GetMappingInfos();
	    	    
		uint16_t __ComputeMappingQual( Hsp* first, Hsp* second, uint32_t nb_subopthits);
	    uint16_t __BwaLikeApproxMappingQual();
};

/**
 * \brief the GlintEngineDouble 
 *         - The class represents the atomic behavior of the glint process, used with paired_end
*/
class GlintEngineDouble: public GlintEngineBase
{	
	public:
		GlintEngineDouble(QueryBufferDouble* q, Display& d):
			GlintEngineBase(q,d),
			__fwd(&(q->GetFwdQueryBuffer()),d),
			__bwd(&(q->GetBwdQueryBuffer()),d)
			{ };
		GlintEngineDouble(const GlintEngineDouble& eng):
			GlintEngineBase(eng),
			__fwd(&(((QueryBufferDouble*)_query_ptr)->GetFwdQueryBuffer()),_display),
			__bwd(&(((QueryBufferDouble*)_query_ptr)->GetBwdQueryBuffer()),_display)
			{ };
		virtual ~GlintEngineDouble(){ };
	    	
	    void BuildHsps ();
	    int  GetNbHsp () const { return __fwd.GetNbHsp() + __bwd.GetNbHsp(); };
	    //int  GetNbSelHsp() const { return __fwd_hspl.size() + __bwd_hspl.size(); };
	    void HspReset(){ __fwd.HspReset(); __bwd.HspReset(); __matepairs.clear(); };
	    
	      	
	protected:
		
		bool _PotentialHspPair() const { return (__fwd.GetNbHsp()>0 && __bwd.GetNbHsp()>0); };
		//bool _OverlappingHsps(Hsp* i, Hsp* j);
		
	protected:

		void __DisplayAll();	
		void __DisplayBestHspPair();
		void __DisplayAllHspPairs();
		void __BuildHsps () { __fwd.BuildHsps(); __bwd.BuildHsps(); };		
		void __RecordHsps();
		void __SortingDoubleHsps(pt2CmpFunc cmp_fn_ptr);			
		bool __ArePaired(const Hsp*, const Hsp*);
		void __GetMappingInfos();
		
		virtual void __IdentifyPairedHsps();
		
		virtual GlintEngineDouble* Clone() { return new GlintEngineDouble( *this ); };
					
		vector<MatePair> __matepairs;
				
		GlintEngine __fwd;
		GlintEngine __bwd;	
};

/**
 * \brief the GlintEngineDoublePCR 
 *         - The class represents the atomic behavior of the glint process when emulating re-PCR
*/
class GlintEngineDoublePCR: public GlintEngineDouble
{	
	public:
		GlintEngineDoublePCR(QueryBufferDouble* q, Display& d):GlintEngineDouble(q,d) {};
	
	protected:
	void __IdentifyPairedHsps();
	bool _OverlappingHsps(Hsp* i, Hsp* j);
	
	
};

#endif
