/*
   $Id: GlintEngineFreq.h 649 2011-10-31 13:57:42Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#ifndef __GLINTENGINEFREQUENCY_H__
#define __GLINTENGINEFREQUENCY_H__

#include "buffer.h"
#include "GlintEngine.h"
#include "Display.h"
#include <stack>

enum FREQ
	{
	LOW = 0,
	HIGH = 1,
	CONDEMNED = 2
	};

typedef unsigned char frequence;


/** \brief the Comparison hierarchy 
 *         - The aim is to generate design oligonucleotide primer
 *				for FISH experiment
 * 		   - First of all we generate an Index Tree
 * 		   - Then we select non frequent probes from a query
 * 		   - A second selection gives regions suitable to be hybridization probes.
 * 		   - Finally we select primers for PCR considering Tm and %GC
*/


/** \brief The base abstract object
*/
class GlintEngineFreq: public GlintEngine {
	
	public :
		//Constructeur
		GlintEngineFreq(QueryBuffer*, Display& d);
		//Destructeur
		~GlintEngineFreq()
		{
			if( frequentBuffer != NULL) delete frequentBuffer;
		}
		
		/*-------- GENERAL -----------*/
		private :
		size_t                seq_length;
		uint8_t                 msk_len;
		const unsigned char*  seq_base;
		Buffer*               frequentBuffer;     // Buffer containing the recoded sequence : LOW, HIGH or CONDEMNED (see enum FREQ)	
		
		public :
		size_t       GetBufferSize()     { return frequentBuffer->size(); };
		Buffer*      GetFrequentBuffer() const{return frequentBuffer;};
		frequence*   GetBufferBase()     { return (frequence*) frequentBuffer -> GetBase(); };
		
		unsigned char 	Decode(frequence f);

		//Display
		void _DisplayBuffer         ( const QueryBuffer& query );
		void _DisplayFasta          ( const QueryBuffer& query, uint32_t start, uint32_t end );
		void _DisplaySuitableFasta  ( const QueryBuffer& query );
		void _DisplayFastaATGCN     ( const QueryBuffer& query );
		
		protected : 
		bool _BufferPut(frequence f) { return frequentBuffer->Put((char) f); };
		
		/*-------- MERMATCH -----------*/
		public :
		uint32_t GetNbTotalPos() const {return nbTotalPos;};
		void BuildHsps();
		void Run();
		
		private :
		void __SearchingForSeeds();
		
		private :
		mutable uint32_t nbTotalPos;
		
		/*-------- TOLERANCE -----------*/
		
		public :
		stack<regionFreq>           GetSuitableRegions() const{return SuitableRegions;};
		
		private :
		double                  CondemnationScore[3];
		double                  SuitableScore[3];

		stack<regionFreq>           CondemnedRegions;
		stack<regionFreq>           SuitableRegions;
		
		void IdentifyCondemnedZone();
		void IdentifySuitableZone();
		void SetCondemnedZone();
};

inline unsigned char GlintEngineFreq::Decode(frequence f)
{
	unsigned char c;
	
	switch ( f ) 
	{
		case HIGH      : c = 'H'; break;
		case LOW       : c = 'L'; break;
		case CONDEMNED : c = 'C'; break;
		default        : c = '?'; break;
	}
	
	return c;
}

#endif
