/*
   $Id: IndexTree.cpp 649 2011-10-31 13:57:42Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

using namespace std;
#include <iostream>
#include <deque>

#include <math.h>
#include <stdlib.h>
#include <assert.h>
#include "IndexTree.h"
#include "ptypes.h"

numeric_limits<uint8_t> limit_uint8_t;
numeric_limits<uint32_t> limit_uint32_t;
numeric_limits<uint64_t> limit_uint64_t;

extern RealLogger* logger;

template <class T> T * SingleLinkedList<T>::PopBack() {
    if (IsEmpty())
        return NULL;

    T * rvl = first;
    first = first->brother;
    if (first == NULL)
        last = NULL;          // Empty object

    return rvl;
}

template <class T> void SingleLinkedList<T>::PushFront(T * p) {
    if (p==NULL) return;

    p->brother=NULL;        // Pushing only one element, marking the end of the chain
    if (IsEmpty())
    {
        first = p;
    }
    else
    {
        last->brother = p;
    };
    last = p;
    return;
}

template <class T> void SingleLinkedList<T>::PushFrontList(T * p) {
    if (p==NULL) return;

    if (IsEmpty())
    {
        first = p;
        last = p;
    }
    else
    {
        last->brother = p;
    };

    while(p->brother!=NULL)     // Pushing a whole chain, many elements together
    {
        p=p->brother;
    }
    last = p;

    return;
}

/*
 Name:      IndexTree::IndexTree (THE CONSTRUCTOR)
 Usage:     IndexTree t(&d)
 Function:  initialize a IndexTree
            The top-level array is initialized
*/

IndexTree::IndexTree(): 
	__parameters(Parameters::Instance()),
	__bank(Bank::Instance()),
	__mask(Mask::Instance()),
	offset(0),
	ct_pass(1),
	inf_code(0) 
{

	/* Initializing the number of nodes = depends on the mask */
	uint8_t n = __mask.GetLength(ROOT_LEVEL);
	nb_top=1<<2*n;     // 4**n

	/* Initializing the top nodes */
	top = new Node[nb_top];
	for(uint32_t i=0; i<nb_top; i++)
		top[i].hash_code = i;

	/* Initializing the number of passes from bank */
	SetNbPass(__parameters.GetIndexNbPass());

	/* Initializing the serialize_shift table (see the bank definition) */
	for (size_t i=0;i<=9;i++) {
		switch(i) {
			case 4:	serialize_shift[i] = new SerializeShift<uint16_t>; break;
			case 5:
			case 6:	serialize_shift[i] = new SerializeShift<uint32_t>; break;
			case 9:	serialize_shift[i] = new SerializeShift<uint64_t>; break;
			default:  serialize_shift[i] = NULL; break;
		}
	}

	/* Initializing the copy of the masque's leaf level */
	leaf_level = __mask.GetLeafLevel();
}

IndexTree::~IndexTree()
{
	for (size_t i=0;i<=9;i++)
	{
		if ( serialize_shift[i] != NULL )
		{
			delete serialize_shift[i];
		}
	}
	delete[] top;
}

void IndexTree::__FreeAll() {
    MemoryPool::Instance().FreeAll();  // Deleting the nodes alltogether
    Node::Clear();                      // Resetting the memory consumption indicators in Node and Leaf
    Leaf::Clear();
    Pos::Clear();
    for(uint32_t i=0; i<nb_top; i++)         // Top nodes are not deleted, we just reset their sons
        top[i].son = NULL;
}

 /**
  * \brief Initialize the data structures for setting the pass numbers
  * \param n The number of passes to set
  */

void IndexTree::SetNbPass(uint8_t n)
{
    nb_pass = n;
    ct_pass = 1;
    lg_pass = nb_top/n;
    inf_code = 0;
    sup_code = lg_pass;
    offset = 0;
    top_info.clear();
    __FreeAll();
}

/**
* \brief Increment the pass number, if possible. 
*        If the passe number is incremented, all the memory pools are freed
* \return The pass number, 0 if we are already at the last pass, so nothing was done
*/

uint8_t IndexTree::NextPass()
{
    if (ct_pass!=nb_pass)
    {
        ct_pass++;
        inf_code = sup_code;
        sup_code = (ct_pass==nb_pass) ? nb_top : inf_code + lg_pass;
        __FreeAll();
        return ct_pass;
    }
    else
        return 0;
}

/**
 * \brief Checks if the hash code of the window at level ROOT_LEVEL is in the current interval 
 *        (the interval depends on the pass number)
 * \param w The window to check
 * \return true/false
 */

 bool  IndexTree::Phylum(const Window &w) const {
     uint32_t code=w.TreeHash(ROOT_LEVEL);
     return (code>=inf_code && code <sup_code);
 }

/*========================================================================
           Inserting a leaf in the tree, from a word included in a window
==========================================================================*/

/**
 * \brief Insert a leaf in the tree, or retrieve the leaf if already inserted.\n
 *        If the hash_code at the root level is outside the current pass, NULL is returned
 * \param w The window encapsulating the word to insert
 * \return The inserted or retrieved leaf, or NULL if the phylum is outside the current interval
 */

Leaf* IndexTree::Insert(const Window & w)
{
    if (Phylum(w))
    {
        uint32_t code=w.TreeHash(ROOT_LEVEL);
        Node* father = top+code;
        for (uint8_t l=ROOT_LEVEL+1;l<leaf_level;l++)
        {
            code = w.TreeHash(l);
            Node* new_node;
            if (father->son!=NULL)
            {
                Node* strt = (Node*)father->son;
                Node* prev;
                new_node = __HorizNodeSearch(strt,code,&prev);
                if (new_node==NULL)                 // Not found - We must create this node
                {
                    if (prev==NULL)                 // Must be created at front of list
                        new_node = new Node(code,father,1);
                    else
                        new_node = new Node(code,prev);
                };
            }
            else
            {
                new_node = new Node(code,father,1); // New branch created
            }
            father = new_node;                      // for the next iteration
        };

        Leaf* strt = (Leaf*) father->son;
        code = w.TreeHash(leaf_level);
        Leaf* prev;
        Leaf* leaf = __HorizLeafSearch(strt,code,&prev);
        if (leaf == NULL)
        {
            if (prev==NULL)
                leaf = new Leaf(code,father,1);
            else
                leaf = new Leaf(code,prev);
            __bank.IncrNbWords();
        }
        return leaf;
    }
    else
    {
        return NULL;
    }
}

/*========================================================================
           Deleting a leaf of the tree, from a word included in a window
==========================================================================*/

/**
 * \brief Delete a leaf from the tree.\n
 *        If possible, delete also every intermediate nodes
 * \param w The window encapsulating the word to delete
 * \return 0 if the leaf was successfulyy deleted, 1 if the leaf was not in the tree
 */
int IndexTree::Delete (const Window& w)
{
	uint32_t code=w.TreeHash(ROOT_LEVEL);
    if (Phylum(w) && top[code].son!=NULL) {    // If there is something in this phylum, search here
		return __Delete(top+code,w,ROOT_LEVEL+1);
	} else {                                // Else, nothing found
        return 1;
	};
}

/**
 * \brief Delete a leaf, starting the path to p's son.\n
 *        All the intermediate nodes will be deleted if possible.
 * \param p The father of the starting node
 * \param w: The window encapsulating the word to delete
 * \param level: The level at which the delete is started
 */
int IndexTree::__Delete(Node * father, const Window& w, uint8_t level)
{
    int rvl=0;
    uint32_t code = w.TreeHash(level);
    if (level<leaf_level)              // We have nodes at this level
    {
        Node* p = (Node*)father->son;
        Node* q;
        Node* r = __HorizNodeSearch(p,code,&q);
        if (r!=NULL)                // The node was found (should be)
        {
            rvl = __Delete(r,w,level+1);  // Recursive call for the next level
            if (rvl !=0)
                return rvl;         // An error was found under
            if(r->son==NULL)  // This node does not have any son, we delete it
            {
                if (q!=NULL)
                    q->brother=r->brother;
                else
                    father->son=r->brother;    // May be NULL if last node for this branch
                delete r;
            }
        }
        else
        {
            rvl = 1;
        };
    }
    else
    {
        Leaf* p = (Leaf*)father->son;
        Leaf* q;
        Leaf* r = __HorizLeafSearch(p,code,&q);
        if (r!=NULL)                        // The leaf was found (should be): delete it
        {
            if (q!=NULL)
                q->brother=r->brother;
            else
                father->son=r->brother;
            delete r;
            __bank.DecrNbWords();
        }
        else
        {
            rvl = 1;
        };
    }
    return rvl;
}

/**
 * \brief Add a position to the leaf, allocating memory if necessary
 * \param new_pos The position to add
 * \param min_dist The minimum distance between the new position and the last position already stored, to avoid overlap.
 * \return The new positions count
 */
uint8_t Leaf::NewPos(uint32_t new_pos, uint8_t min_dist) {
    if (pos==NULL) {
        pos = new Pos(new_pos);
        nb_pos=1;
    }
    else
    {
        /* Find the last position recorded */
        Pos* last_pos=pos;
        int i;
        for(i=1; i< nb_pos; i++)
            last_pos = last_pos->brother;
        //cerr << "coucou " << i << '\n';
        /* Append the new position only if there is no overlap */
        if (new_pos-last_pos->position > min_dist)
        {
            last_pos->brother = new Pos(new_pos);
            nb_pos++;
#ifdef DEBUGLOG
            logger->SetLogType("T") << "Adding a new position to the tree - pos= " << new_pos << " nb of pos= " << pos << "\n";
#endif
        }
        else
        {
#ifdef DEBUGLOG
            logger->SetLogType("T") << "Overlapping here: " << last_pos->position << "...." <<  new_pos << " (min " << (int) min_dist << ")\n";
#endif
        }
    };

	assert(nb_pos<=limit_uint8_t.max());

    return nb_pos;
}


/*========================================================================
           Searching a leaf in the tree, from a word included in a window
==========================================================================*/

/**
 * \brief Search a leaf in the tree. Use the TreeHash of the word at root level to know the phylum, then call __Search
 * \param w The window encapsulating the word to delete
 * \return The leaf of the searched word, or NULL if not found (may be it is not in the current interval)
 */

Leaf* IndexTree::Search (const Window& w)
{
	uint32_t code=w.TreeHash(ROOT_LEVEL);
    if (Phylum(w) && top[code].son) {           // If there is something in this phylum, search here
		return __Search(top+code,w,ROOT_LEVEL+1);
	} else {                                // Else, nothing found
		return NULL;
	};
}

/**
 * \brief Search a leaf from a known phylum.
 * \param p The father of the starting node.
 * \param w The window encapsulating the searched word.
 * \param level The level of the tree.
 * \return A pointer to the found node, of NULL if not found
 */

Leaf* IndexTree::__Search(Node* p, const Window& w, uint8_t level)
{
    void* n = p->son;
    uint32_t code = w.TreeHash(level);
    if (level==leaf_level)        // It is a leaf, end of recursivity
    {
        return __HorizLeafSearch((Leaf*)n,code);
    }
    else
    {
        n=__HorizNodeSearch((Node*)n,code);
        if (n==NULL)
            return (Leaf*)n;
        else
            return __Search((Node*)n,w,level+1);
    }
}

/**
 * \brief Search a node whose hash_code is known.
 * \param p The starting node.
 * \param code The hash code.
 * \return A pointer to the found node, of NULL if not found
 */
Node* IndexTree::__HorizNodeSearch(Node * p, uint32_t code)
{
    for(Node* n=p;n != NULL; n=n->brother)
    {
        if (code==n->hash_code)
            return n;
        if (code<n->hash_code)
            return NULL;
    };
    return NULL;
}

/**
 * \brief Search a node whose hash_code is known (2nd version)
 * \param p The starting node.
 * \param code The hash code.
 * \param[out] prev The node just before the found node (useful for deleting the found node), 
 *                  or the node at the insert position (useful for inserting a new node).
 * \return A pointer to the found node, of NULL if not found
 */
Node* IndexTree::__HorizNodeSearch(Node * p, uint32_t code, Node ** prev)
{
    *prev=NULL;
    for(Node* n=p;n != NULL; n=n->brother)
    {
        if (code==n->hash_code)
            return n;
        if (code < n->hash_code)
            return NULL;
        *prev = n;
    };
    return NULL;
}

/**
 * \brief Search a leaf whose hash_code is known.
 * \param p The starting node.
 * \param code The hash code.
 * \return A pointer to the found leaf, of NULL if not found
 */
Leaf* IndexTree::__HorizLeafSearch(Leaf * p, uint32_t code)
{
    for(Leaf* n=p;n != NULL; n=n->brother)
    {
        if (code==n->hash_code)
            return n;
        if (code < n->hash_code)
            return NULL;
    };
    return NULL;
}

/**
 * \brief Search a leaf whose hash_code is known (2nd version)
 * \param p The starting node.
 * \param code The hash code.
 * \param[out] prev The leaf just before the found leaf (useful for deleting the found leaf), 
 *                  or the leaf at the insert position (useful for inserting a new leafs).
 * \return A pointer to the found leaf, of NULL if not found
 */
Leaf* IndexTree::__HorizLeafSearch(Leaf * p, uint32_t code, Leaf ** prev)
{
    *prev=NULL;
    for(Leaf* n=p;n != NULL; n=n->brother)
    {
        if (code==n->hash_code)
            return n;
        if (code < n->hash_code)
            return NULL;
        *prev = n;
    };
    return NULL;
}


/*=====================================================================
       Printing the tree
=======================================================================*/

/**
 * \brief Print formatted the complete tree, mainly for debugging purposes
 */

ostream & operator<<(ostream& os, const IndexTree& t) {
    for (uint32_t code=0;code<t.nb_top;code++)
        t.top[code].Print(os,t.__mask);
    return os;
}

/**
 * \brief If the node does not have any son, return\n
 *        Else print the node, then its sons, then its brothers.
 * \param os The output stream
 * \param m The mask (used for reconstruction of the word)
 * \param level The level at which the node lives in the tree (default=ROOT_LEVEL)
 * \param word The reconstructed word (default = "")
 * \param indent The number of spaces to insert before any printed character
 */
void Node::Print(ostream& os, const Mask& m, uint8_t level, const string& word, int indent)
{
    if (son==NULL)
        return;

    string begin="";
    for(int i=0;i<indent;i++) begin+=' ';

    // First, print the node itself
    string reconstructed = m.Reconstruct(word,hash_code,level);
    os << begin << reconstructed << ' ' << hex << hash_code << " (" << (int)level << '/' << m.GetLeafLevel() << ") \n";

    // Second, print the son - It should work

    if (m.GetLeafLevel()-level>1)     // The son is still a Node*
    {
        if (son != NULL)
        {
            Node* s = (Node*) son;
            s->Print(os,m,level+1,reconstructed,indent+m.GetLength()-1);
        }
        else
        {
            os << begin << "THERE SHOULD BE A NODE HERE " << (int)level+1 << '/' << m.GetLeafLevel() << '\n';
        }
    }
    else                        // The son is a Leaf*
    {
        if (son != NULL)
        {
            Leaf *s = (Leaf*) son;
            s->Print(os,m,reconstructed,indent+m.GetLength()-1);
        }
        else
        {
            os << begin << "THERE SHOULD BE A LEAF HERE " << (int)level+1 << '/' << m.GetLeafLevel() << '\n';
        }
    }

    // Third, and if possible only, print the brother

    if (brother != NULL)
    {
        brother->Print(os,m,level,word,indent);
    };
}

/**
 * \brief Print the leaf, then its brothers
 * \param os The output stream
 * \param m The mask (used for reconstruction of the word)
 * \param word The reconstructed word (default = "")
 * \param indent The number of spaces to insert before any printed character
 */
void Leaf::Print(ostream& os, const Mask& m, const string& word, int indent)
{
    // First, print the leaf
    uint8_t level = m.GetLeafLevel();     // The leaves live only at this level
    string begin="";
    for(int i=0;i<indent;i++) begin+=' ';
    string reconstructed = m.Reconstruct(word,hash_code,level);
    os << begin << reconstructed << ' ' << hex << hash_code << " (" << (int)level << '/' << m.GetLeafLevel() << ") ";
    os << dec << GetNbPos() << " positions - ";
    if (pos !=NULL) {
        Pos* p = pos;
        int dsp_pos = (nb_pos < 10) ? nb_pos : 10;     // Do not display all positions
        for (int i=0;i<dsp_pos;i++) {
            os << dec << p->position << ' ';
            p=p->brother;
        };
        if (dsp_pos==nb_pos)
        {
            os << '\n';
        }
        else
        {
            os << "...\n";
        };
    };

    // Second, print the  brothers, if any
    if (brother != NULL)
    {
        brother->Print(os,m,word,indent);
    };
}


/*            Version avec compression                */

/*
 Name:      IndexTree::WriteCompressed (2 versions)
 Usage:     int j = tree.WriteCompressed(pf)
            int j = tree.Writecompressed(m)
 Function:  Write the tree to the opened file descriptor pf /to the Buffer m
 Arg:       pf = A file descriptor
            m  = A Buffer
 Return:    The number of phyla written
 Global:
*/

int IndexTree::WriteCompressed(FILE * pf)
{
    //cerr << inf_code << ' ' << sup_code-1 << '\n';
    for (uint32_t i=inf_code; i< sup_code; i++)
    //for (i=0; i< 2; i++)  // for debugging only
    {
        if (top[i].son != NULL)
        {
			TopInfo e;
			e.sons = __WriteSons(pf,top[i].son,ROOT_LEVEL+1);
	    	assert((offset - __bank.GetNodeWidth(ROOT_LEVEL+1))<limit_uint64_t.max());
	    	e.start = offset - __bank.GetNodeWidth(ROOT_LEVEL+1);
			top_info.push_back(e);
        }
        else
        {
            top_info.push_back(enul);
        }
    };

    // We now write all top levels, but ONLY if we are working in the last pass

    if (ct_pass==nb_pass)
    {
        while(!top_info.empty())
        {
            TopInfo e = top_info.back();
            top_info.pop_back();
            uint8_t s = e.sons;
            size_t t = e.start;
            offset += sizeof(uint8_t)*fwrite(&s, sizeof(uint8_t), 1, pf);
			offset += __CheckAndWriteOffset(pf,t,ROOT_LEVEL);
        };
       cout << "INDEX SIZE " << offset << "\n"; 
    };
    __bank.Size = offset;
    
    return nb_top;
}

int IndexTree::WriteCompressed(Buffer* m)
{
    for (uint32_t i=inf_code; i< sup_code; i++)
    //for (i=0; i< 2; i++)  // for debugging only
    {
        if (top[i].son != NULL)
        {
            TopInfo e;
            e.sons = __WriteSons(*m,top[i].son,ROOT_LEVEL+1);
			assert((offset - __bank.GetNodeWidth(ROOT_LEVEL+1))<limit_uint64_t.max());
            e.start = offset - __bank.GetNodeWidth(ROOT_LEVEL+1);
            top_info.push_back(e);
        }
        else
        {
            top_info.push_back(enul);
        }
    };

    // We now write all top levels, but ONLY if we are working in the last pass

    if (ct_pass==nb_pass)
    {
        while(!top_info.empty())
        {
            TopInfo e = top_info.back();
            top_info.pop_back();
            uint8_t s = e.sons;
            size_t t = e.start;
            offset += sizeof(uint8_t) * (uint8_t) m->Put((char)s);
            offset += __CheckAndWriteOffset(*m,t,ROOT_LEVEL);

        };
    };
    __bank.Size = offset;
    return nb_top;
}
/*
 Name:      IndexTree::__WriteSons (2 versions)
 Usage:     int son = __WriteSons(f,strt,level)
            int son = __WriteSons(m,strt,level)
 Function:  Write the subtree starting at strt (with level level) to the file f. / the Buffer m
            If level < GetLeafLevel(), strt is a node and WriteSons is called first for the son, second for the brother
	        If level == GetLeafLevel(), strt is a leaf: the positions are written, the WriteSons is called for the brother
            The private member offset is updated
 Arg:       f     = A file descriptor
            m     = A Buffer
            strt  = The subtree to write (if NULL, nothing is written)
	        level = The level at which strt lives
 Return:    The number of brothers at the current level
 Global:    Tree_var, Masque
*/

uint8_t IndexTree::__WriteSons(FILE* pf, void* strt, uint8_t level)
{
	uint8_t brothers=0;
	uint8_t sons=0;
	size_t my_offset;

	if (strt!=NULL){
		if (level<leaf_level) {	/* It is a node */
			Node * n = (Node*) strt;
			sons=__WriteSons(pf,(void*)n->son,level+1);            // First, write the son
            my_offset = offset - __bank.GetNodeWidth(level+1);
            brothers=__WriteSons(pf,(void*)n->brother,level)+1;    // Then, write the first brother

			if (sons)                                                   // Write the node himself
                __SerializeNode(pf,n,sons,offset-my_offset,level);

		} else {						/* It is a leaf */
			Leaf*f=(Leaf*) strt;
			if (!f->IsEmpty()) {        // If there are positions for this leaf
				__bank.Positions+=f->GetNbPos();
				my_offset=offset;
				__WriteLeafPositions(pf,f);
				brothers=__WriteSons(pf,(void*)f->brother,level)+1;
				__SerializeLeaf(pf,f,offset-my_offset,level);
			}
		}

		return brothers;
	}
    else
    {
        return 0;
    };
}

uint8_t IndexTree::__WriteSons(Buffer& m, void* strt, uint8_t level)
{
	uint8_t brothers=0;
	uint8_t sons=0;
	size_t my_offset;

	if (strt!=NULL){
		if (level<leaf_level) {	/* It is a node */
			Node * n = (Node*) strt;
			sons=__WriteSons(m,(void*)n->son,level+1);            // First, write the son
            my_offset = offset - __bank.GetNodeWidth(level+1);
            brothers=__WriteSons(m,(void*)n->brother,level)+1;    // Then, write the first brother

			if (sons)                                                   // Write the node himself
                __SerializeNode(m,n,sons,offset-my_offset,level);

		} else {						/* It is a leaf */
			Leaf*f=(Leaf*) strt;
			if (!f->IsEmpty()) {        // If there are positions for this leaf
				__bank.Positions+=f->GetNbPos();
				my_offset=offset;
				__WriteLeafPositions(m,f);
				brothers=__WriteSons(m,(void*)f->brother,level)+1;
				__SerializeLeaf(m,f,offset-my_offset,level);
			}
		}
		return brothers;
	}
    else
    {
        return 0;
    };
}

/*
 Name:      IndexTree::__SerializeNode (2 versions)
 Usage:     __SerializeNode(f,node,sons,shift_count,depth)
            __SerializeNode(m,node,sons,shift_count,depth)
 Procedure: Serialize and write a node to the descriptor f  / the Buffer m
 Arg:       f     = A file descriptor
            m     = A Buffer
            node  = A node
			sons  = The number of sons this node has
	        shift_count = A shift count to write
			depth = The depth of the node in the tree
 Global:    Tree_var
*/

void IndexTree::__SerializeNode(FILE *pf,Node* n, uint8_t sons, size_t shift, uint8_t level)
{
    uint8_t code = (uint8_t) n->hash_code;

	offset += sizeof(uint8_t)*fwrite(&code, sizeof(uint8_t), 1, pf);
	offset += sizeof(uint8_t)*fwrite(&sons, sizeof(uint8_t), 1, pf);
    offset += __CheckAndWriteOffset(pf,shift,level);
}

void IndexTree::__SerializeNode(Buffer& m,Node* n, uint8_t sons, size_t shift, uint8_t level)
{
    uint8_t code = (uint8_t) n->hash_code;
    offset += sizeof(uint8_t) * (uint8_t) m.Put((char)code);
    offset += sizeof(uint8_t) * (uint8_t) m.Put((char)sons);
	offset += __CheckAndWriteOffset(m,shift,level);
}

/*
 Name:      Tree::__SerializeLeaf (2 versions)
 Usage:     __SerializeLeaf(f,leaf,shift_count)
 Procedure: __Serialize and write a leaf to the descriptor pf / the Buffer m
 Arg:       pf    = A file descriptor
            m     = A Buffer
            l     = A leaf
	        shift = A shift count to write
			level = The depth of the leaf in the tree
 Global:    Tree_var
*/

void IndexTree::__SerializeLeaf(FILE *pf, Leaf* l, size_t shift, uint8_t level)
{
    uint8_t code   = (uint8_t) l->hash_code;
    uint8_t nb_pos = (uint8_t) l->GetNbPos();
    offset += sizeof(uint8_t)*fwrite(&code, sizeof(uint8_t), 1, pf);
	offset += sizeof(uint8_t)*fwrite(&nb_pos, sizeof(uint8_t), 1, pf);
    offset += __CheckAndWriteOffset(pf,shift,level);
}

void IndexTree::__SerializeLeaf(Buffer &m, Leaf* l, size_t shift, uint8_t level)
{
    uint8_t code   = (uint8_t) l->hash_code;
    uint8_t nb_pos = (uint8_t) l->GetNbPos();
    offset += sizeof(uint8_t) * (uint8_t) m.Put((char)code);
    offset += sizeof(uint8_t) * (uint8_t) m.Put((char)nb_pos);
	offset += __CheckAndWriteOffset(m,shift,level);
}

/*
 Name:      Tree::__WriteLeafPositions (2 versions)
 Usage:     __ WriteLeafPositions(f,leaf)
            __ WriteLeafPositions(m,leaf)
 Function:  Write the array of leaf positions to the descriptor f / the Buffer m
 Arg:       pf    = A file descriptor
            m     = A Buffer
            f     = A leaf
 Global:    Tree_var
 Return:    The number of bytes written
*/

int IndexTree::__WriteLeafPositions(FILE* pf, Leaf* f)
{
    int bytes=0;
    Pos* p=f->pos;
    //cerr << "P ";
    for (int i=0;i<f->nb_pos;i++) {
        //cerr << p->position << ' ';
		assert(p->position<=__bank.GetLength());
        bytes += sizeof(uint32_t) * fwrite((void *)&(p->position),sizeof(uint32_t),1,pf);
        p=p->brother;
    }
    offset += bytes;
    return bytes;
    //cerr << '\n';
}

int IndexTree::__WriteLeafPositions(Buffer& m, Leaf* f)
{
    int bytes=0;
    Pos* p=f->pos;
    //cerr << "P ";
    for (int i=0;i<f->nb_pos;i++) {
        //cerr << p->position << ' ';
        bytes += sizeof(uint32_t) * m.Put(p->position);
        p=p->brother;
    }
    offset += bytes;
    return bytes;
    //cerr << '\n';
}


/*
 Name:      IndexTree::__CheckAndWriteOffset  (2 versions)
 Usage:     __CheckAndWriteOffse(pf,shift)
            __CheckAndWriteOffse(m,shift)
 Function:  Write the array of leaf positions to the descriptor f / the Buffer m
 Arg:       pf    = A file descriptor
            m     = A Buffe
            shift = A shift count to write
			level = The depth in the tree
 Global:    Tree_var
 Return:    The number of bytes written
*/

int IndexTree::__CheckAndWriteOffset(FILE* pf,size_t shift,uint8_t level)
{
	BaseSerializeShift &fct = *serialize_shift[__bank.GetNodeWidth(level)];
    uint8_t ofs = fct(pf,shift);    // Calling the good SerializeShift function
    if (ofs != 0)
    {
        return ofs;
    }
    else
    {
        cerr << shift << " out of range at level "<<(int)level<<" (see manual for details)\n";
        exit(1);
    };

}

int IndexTree::__CheckAndWriteOffset(Buffer& m,size_t shift,uint8_t level)
{
	BaseSerializeShift &fct = *serialize_shift[__bank.GetNodeWidth(level)];
    uint8_t ofs = fct(m,shift);    // Calling the good SerializeShift function
    if (ofs != 0)
    {
        return ofs;
    }
    else
    {
        cerr << shift << " out of range at level "<<(int)level<<" (see manual for details)\n";
        exit(1);
    };


}

/* Creating the static members of those objects */

size_t NBaseNode::mem_used = 0;
SingleLinkedList<Node> NBaseNode::reserve;

size_t NBaseLeaf::mem_used = 0;
SingleLinkedList<Leaf> NBaseLeaf::reserve;

size_t NBasePos::mem_used = 0;
SingleLinkedList<Pos> NBasePos::reserve;


