/*
   $Id: SeqInfo.cpp 649 2011-10-31 13:57:42Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#include "utils.h"
#include "SeqInfo.h"

#define INDEX_VERSION "1.0"

/**
 * \brief Building a SeqInfo from a file name: If the file name is called bank.fasta, we read the SEQUENCE = lines
 *        on the file bank.fasta.he, and the class is initialized from this file
 *        If the file "" is specified, we gently return, without initializing anything
 * \todo  UTILISER LA LIGNE SEQ POUR CONNAITRE ET VERIFIER LA COHERENCE DU FICHIER (nb de lignes)

 */
SeqInfo::SeqInfo (const string& index_name, CompressionAlgorithm compression) : __old_index(false)
{
	if ( index_name == "")
		return;
				
	string col1,tmp;
	IstreamPtr is_ptr(index_name, HE_EXTENSION,compression);
	istream& is = *is_ptr;

	while ( true )
	{
		SequencePosition sp;
		is >> col1;
		if ( is.eof() || !is )
			break;
		if (col1 != "SEQUENCE")
			getline(is,tmp); // skip the end of line
		else
		{
			is >> sp;
			sequence_positions.push_back(sp);
		}
	}

	if ( sequence_positions.begin()->end== 0 )
		__old_index = true;
}

/**
 * \brief SequencePosition output operator
 */
ostream & operator<<(ostream& os, SequencePosition const & sp) {
    os << sp.start << '-' << sp.end << ' ' << sp.Name;
    return os;
}

/**
 * \brief SequencePosition input operator
 *        File format: START NAME (old indexes)
 *                 or: START-END NAME (new indexes)
 */
istream& operator>>(istream& is, SequencePosition& sp) {
	string boundaries;	
    is >> boundaries >> sp.Name;
    size_t sign_position = boundaries.find('-');
    if ( sign_position == string::npos )
    {
		sp.start = atoi(boundaries.c_str());
	}
	else
	{
		sp.start = atoi(boundaries.substr(0,sign_position).c_str());
		++sign_position;
		sp.end   = atoi(boundaries.substr(sign_position,boundaries.size() - sign_position ).c_str());
	}
    return is;
}

/**
 * \brief Init the __rev field
 */
ReverseComplement::ReverseComplement()
{
	for (int i=0;i<256;i++) 
	{
		__rev[i] = i;
	}
	__rev['A'] = 'T';
	__rev['T'] = 'A';
	__rev['C'] = 'G';
	__rev['G'] = 'C';
	__rev['a'] = 't';
	__rev['t'] = 'a';
	__rev['c'] = 'g';
	__rev['g'] = 'c';
	__rev['N'] = 'N';
}

/**
 * \brief Reverse complements the sequence, working inplace
 */
void ReverseComplement::operator()( unsigned char* s, uint32_t len )
{
	if ( len>0 )
	{
		for (uint32_t i=0; i<len/2; ++i)
		{
			char tmp   = __rev[(uint8_t)s[i]];
			s[i]       = __rev[(uint8_t)s[len-1-i]];
			s[len-1-i] = tmp;
		}
		if ( len%2 == 1 )
		{
			s[len/2] = __rev[(uint8_t)s[len/2]];
		}
	}
}

/**
 * \brief Reverse complement the sequence, from a string to another
 */
string ReverseComplement::operator()(const string & seq)
{
	string reversed;
	if ( seq.length() > 0 )
	{
		for ( string::size_type i=seq.length(); i>0; )
		{
			reversed += __rev[(uint8_t)seq[--i]];
		}
	}
	return reversed;
}

void Reverse(unsigned char* s, uint32_t len )
{
	if ( len>0 )
	{
		for ( uint32_t i=0, j=len-1; i<len/2;i++,j--)
		{
			char tmp = s[i];
			s[i] = s[j];
			s[j] = tmp;
		}
	}
}

string Reverse( const string& seq)
{
	string reversed;
	if ( seq.length() > 0 )
	{
		for ( string::size_type i=seq.length(); i>0; )
		{
			reversed += seq[--i];
		}
	}
	return reversed;
}
